package org.amse.ak.schemebuilder.builder;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import org.amse.ak.schemebuilder.model.IBlock;
import org.amse.ak.schemebuilder.model.IBlockFactory;
import org.amse.ak.schemebuilder.model.IDoBlock;
import org.amse.ak.schemebuilder.model.IForBlock;
import org.amse.ak.schemebuilder.model.IIfBlock;
import org.amse.ak.schemebuilder.model.IScheme;
import org.amse.ak.schemebuilder.model.ISwitchBlock;
import org.amse.ak.schemebuilder.model.ITryBlock;
import org.amse.ak.schemebuilder.model.IWhileBlock;
import org.amse.ak.schemebuilder.model.impl.BlockFactory;
import org.sablecc.java.analysis.DepthFirstAdapter;
import org.sablecc.java.node.AAssignmentStatementExpression;
import org.sablecc.java.node.ABeginTag;
import org.sablecc.java.node.ABreakStatement;
import org.sablecc.java.node.ACatchClause;
import org.sablecc.java.node.ACatchTryStatement;
import org.sablecc.java.node.ADefaultSwitchLabel;
import org.sablecc.java.node.AEmptyBasicForStatement;
import org.sablecc.java.node.AEmptyReturnStatement;
import org.sablecc.java.node.AEndTag;
import org.sablecc.java.node.AExpressionBasicForStatement;
import org.sablecc.java.node.AExpressionDoStatement;
import org.sablecc.java.node.AExpressionIfThenElseStatement;
import org.sablecc.java.node.AExpressionIfThenStatement;
import org.sablecc.java.node.AExpressionReturnStatement;
import org.sablecc.java.node.AExpressionSwitchLabel;
import org.sablecc.java.node.AExpressionSwitchStatement;
import org.sablecc.java.node.AExpressionWhileStatement;
import org.sablecc.java.node.AFinally;
import org.sablecc.java.node.AFinallyTryStatement;
import org.sablecc.java.node.AIdentifierBasicForStatement;
import org.sablecc.java.node.AIdentifierDoStatement;
import org.sablecc.java.node.AIdentifierIfThenElseStatement;
import org.sablecc.java.node.AIdentifierIfThenStatement;
import org.sablecc.java.node.AIdentifierReturnStatement;
import org.sablecc.java.node.AIdentifierSwitchLabel;
import org.sablecc.java.node.AIdentifierSwitchStatement;
import org.sablecc.java.node.AIdentifierWhileStatementNoShortIf;
import org.sablecc.java.node.AInstanceCreationStatementExpression;
import org.sablecc.java.node.ALocalVariableDeclarationStatement;
import org.sablecc.java.node.AMethodDeclarator;
import org.sablecc.java.node.AMethodInvocationStatementExpression;
import org.sablecc.java.node.APostDecrementStatementExpression;
import org.sablecc.java.node.APostIncrementStatementExpression;
import org.sablecc.java.node.APreDecrementStatementExpression;
import org.sablecc.java.node.APreIncrementStatementExpression;
import org.sablecc.java.node.APrimitiveExpressionEnhancedForStatement;
import org.sablecc.java.node.APrimitiveIdentifierEnhancedForStatement;
import org.sablecc.java.node.AReferenceExpressionEnhancedForStatement;
import org.sablecc.java.node.AReferenceIdentifierEnhancedForStatement;
import org.sablecc.java.node.ASwitchBlockStatementGroup;
import org.sablecc.java.node.AThrowStatementStatementWithoutTrailingSubstatement;
import org.sablecc.java.node.PAdditionalIdentifier;
import org.sablecc.java.node.PBlockStatement;
import org.sablecc.java.node.PCatchClause;
import org.sablecc.java.node.PDim;
import org.sablecc.java.node.PModifier;
import org.sablecc.java.node.PSwitchLabel;
import org.sablecc.java.node.PTypeComponent;

class BuildSchemeVisitor extends DepthFirstAdapter {
    
    private final IBlockFactory myBlockFactory = new BlockFactory();
    
    private final IScheme myScheme;
    
    private IBlock myCurrentBlock = null;
    
    private boolean myMethodNow = false;
    
    private boolean myTagNow = false;
    
    private BuildSchemeState myState = BuildSchemeState.BEGIN_STATE;
    
    // ,      .
    private boolean myMethodExist = false;
   
    private Stack<IBlock> myBlockStack = new Stack<IBlock>();
    
    public BuildSchemeVisitor(IScheme scheme) {
        myScheme = scheme;
        myCurrentBlock = myScheme.getBeginBlock();
    }
    
    /**
     *   ,     .   ,
     *           
     *  ,         ,
     *  .
     * @return true,      ,   
     *  false.         ,
     *   false.
     */
    public boolean isMethodExist() {
        return myMethodExist;
    }
    
    public void inAMethodDeclarator(AMethodDeclarator node) {
        if (node.getIdentifier() != null) {
            String args;
            if (node.getFormalParameterList() == null) {
                args = " ";
            } else {
                args = node.getFormalParameterList().toString();
            }
            String name = node.getIdentifier().toString() + "(" + args + ") ";
            if (name.equals(myScheme.getMethod().getName())) {
                myMethodNow = true;
                myMethodExist = true;
            } else {
                myMethodNow = false;
            }
        }
    }
    
    private void addBlock(IBlock block) {
        switch (myState) {
        case BEGIN_STATE:
        {
            myCurrentBlock.setNext(block);
            break;
        }
        case YES_BLOCK:
        {
            IIfBlock ib = (IIfBlock) myCurrentBlock;
            ib.setYesBlock(block);
            break;
        }
        case NO_BLOCK:
        {
            IIfBlock ib = (IIfBlock) myCurrentBlock;
            ib.setNoBlock(block);
            break;
        }
        case FIRST_DO_BLOCK:
        {
            IDoBlock dwb = (IDoBlock) myCurrentBlock;
            dwb.setFirstBlock(block);
            break;
        }
        case FIRST_FOR_BLOCK:
        {
            IForBlock fb = (IForBlock) myCurrentBlock;
            fb.setFirstBlock(block);
            break;
        }
        case FIRST_WHILE_BLOCK:
        {
            IWhileBlock wb = (IWhileBlock) myCurrentBlock;
            wb.setFirstBlock(block);
            break;
        }
        case SWITCH_BLOCK:
        {
            ISwitchBlock sb = (ISwitchBlock) myCurrentBlock;
            sb.setBlockForLastCase(block);
            break;
        }
        case TRY_BLOCK:
        {
            ITryBlock tb = (ITryBlock) myCurrentBlock;
            tb.setFirstBlock(block);
            break;
        }
        case FINALLY:
        {
            ITryBlock tb = (ITryBlock) myCurrentBlock;
            tb.setFinally(block);
            break;
        }
        }
        myCurrentBlock = block;
        myState = BuildSchemeState.BEGIN_STATE;
    }
    
    private void addActionBlock(String text) {
        if (myMethodNow && !myTagNow) {
            IBlock actionBlock = myBlockFactory.createActionBlock(text);
            addBlock(actionBlock);
        }    
    }
    
    private void defaultComparentBlockOut() {
        if (myMethodNow && !myTagNow) {
            myState = BuildSchemeState.BEGIN_STATE;
            myCurrentBlock = myBlockStack.pop();
        }
    }
    
    //Action blocks.........................................
    
    public void inAAssignmentStatementExpression(AAssignmentStatementExpression node) {
        addActionBlock(node.getAssignment().toString());
    }
    
    public void inPreIncrementStatementExpression(APreIncrementStatementExpression node) {
        addActionBlock(node.getPreIncrementExpression().toString());
    }
    
    public void inAPreDecrementStatementExpression(APreDecrementStatementExpression node) {
        addActionBlock(node.getPreDecrementExpression().toString());
    }
    
    public void inAPostIncrementStatementExpression(APostIncrementStatementExpression node) {
        addActionBlock(node.getPostIncrementExpression().toString());
    }
    
    public void inAPostDecrementStatementExpression(APostDecrementStatementExpression node) {
        addActionBlock(node.getPostDecrementExpression().toString());
    }
    
    public void inAMethodInvocationStatementExpression(AMethodInvocationStatementExpression node) {
        if (myMethodNow && !myTagNow) {
            IBlock invocationBlock = myBlockFactory.createInvocationBlock(
                    node.getMethodInvocation().toString());
            addBlock(invocationBlock);
        }
    }
    
    public void inAInstanceCreationStatementExpression(AInstanceCreationStatementExpression node) {
        addActionBlock(node.getClassInstanceCreationExpression().toString());
    }
    
    public void inALocalVariableDeclarationStatement(ALocalVariableDeclarationStatement node) {
        if (myMethodNow && !myTagNow) {
            IBlock actionBlock = myBlockFactory.createActionBlock(
                    node.getLocalVariableDeclaration().toString());
            addBlock(actionBlock);
        }
    }
    
    public void outAThrowStatementStatementWithoutTrailingSubstatement(
            AThrowStatementStatementWithoutTrailingSubstatement node) {
        addActionBlock(node.getThrowStatement().toString());
    }
    
    //Return blocks................................................
    
    public void inAEmptyReturnStatement(AEmptyReturnStatement node) {
        if (myMethodNow && !myTagNow) {
            IBlock endBlock;
            endBlock = myBlockFactory.createEndBlock("return");
            addBlock(endBlock);
        }
    }
    
    public void inAExpressionReturnStatement(AExpressionReturnStatement node) {
        if (myMethodNow && !myTagNow) {
            IBlock endBlock;
            if (node.getExpression() == null) {
                endBlock = myBlockFactory.createEndBlock("return");
            } else {
                endBlock = myBlockFactory.createEndBlock("return " 
                        + node.getExpression().toString());
            }
            addBlock(endBlock);
        }
    }
    
    public void inAIdentifierReturnStatement(AIdentifierReturnStatement node) {
        if (myMethodNow && !myTagNow) {
            IBlock endBlock;
            if (node.getIdentifier() == null) {
                endBlock = myBlockFactory.createEndBlock("return");
            } else {
                StringBuffer sb = new StringBuffer();
                sb.append("return ");
                if (node.getIdentifier() != null) {
                    sb.append(node.getIdentifier().toString());
                }
                List<PAdditionalIdentifier> ai = node.getAdditionalIdentifiers();
                if (ai != null) {
                    for (PAdditionalIdentifier pai: ai) {
                        sb.append(pai.toString());
                    }
                }
                endBlock = myBlockFactory.createEndBlock(sb.toString());
            }
            addBlock(endBlock);
        }
    }
    
    //If blocks.............................................................
    
    public void inAExpressionIfThenStatement(AExpressionIfThenStatement node) {
        if (myMethodNow && !myTagNow) {
            String text = "";
            if (node.getExpression() != null) {
                text = node.getExpression().toString();
            }
            IBlock ifBlock = myBlockFactory.createIfBlock(text);
            myBlockStack.push(ifBlock);
            addBlock(ifBlock);
            myState = BuildSchemeState.YES_BLOCK;
        }
    }
    
    public void outAExpressionIfThenStatement(AExpressionIfThenStatement node) {
        defaultComparentBlockOut();
    }
    
    public void inAIdentifierIfThenStatement(AIdentifierIfThenStatement node) {
        if (myMethodNow && !myTagNow) {
            StringBuffer sb = new StringBuffer();
            if (node.getIdentifier() != null) {
                sb.append(node.getIdentifier().toString());
            }
            List<PAdditionalIdentifier> ai = node.getAdditionalIdentifiers();
            if (ai != null) {
                for (PAdditionalIdentifier pai: ai) {
                    sb.append(pai.toString());
                }
            }
            IBlock ifBlock = myBlockFactory.createIfBlock(sb.toString());
            myBlockStack.push(ifBlock);
            addBlock(ifBlock);
            myState = BuildSchemeState.YES_BLOCK;
        }
    }

    public void outAIdentifierIfThenStatement(AIdentifierIfThenStatement node) {
        defaultComparentBlockOut();
    }
    
    public void inAExpressionIfThenElseStatement(AExpressionIfThenElseStatement node) {
        if (myMethodNow && !myTagNow) {
            String text = "";
            if (node.getExpression() != null) {
                text = node.getExpression().toString();
            }
            IBlock ifBlock = myBlockFactory.createIfBlock(text);
            myBlockStack.push(ifBlock);
            addBlock(ifBlock);
        }
    }

    public void outAExpressionIfThenElseStatement(AExpressionIfThenElseStatement node) {
        defaultComparentBlockOut();
    }
    
    public void caseAExpressionIfThenElseStatement(AExpressionIfThenElseStatement node) {
        if (myMethodNow && !myTagNow) {
            inAExpressionIfThenElseStatement(node);
            if (node.getIf() != null) {
                node.getIf().apply(this);
            }
            if (node.getLPar() != null) {
                node.getLPar().apply(this);
            }
            
            if (node.getExpression() != null) {
                node.getExpression().apply(this);
            }
            if (node.getRPar() != null) {
                node.getRPar().apply(this);
            }
            IIfBlock currentIfBlock = (IIfBlock) myCurrentBlock;
            myState = BuildSchemeState.YES_BLOCK;
            if (node.getStatementNoShortIf() != null) {
                node.getStatementNoShortIf().apply(this);
            }
            if (node.getElse() != null) {
                node.getElse().apply(this);
            }
            myCurrentBlock = currentIfBlock;
            myState = BuildSchemeState.NO_BLOCK;
            if (node.getStatement() != null) {
                node.getStatement().apply(this);
            }
            outAExpressionIfThenElseStatement(node);
        }
    }
    
    public void inAIdentifierIfThenElseStatement(AIdentifierIfThenElseStatement node) {
        if (myMethodNow && !myTagNow) {
            StringBuffer sb = new StringBuffer();
            if (node.getIdentifier() != null) {
                sb.append(node.getIdentifier().toString());
            }
            List<PAdditionalIdentifier> ai = node.getAdditionalIdentifiers();
            if (ai != null) {
                for (PAdditionalIdentifier pai: ai) {
                    sb.append(pai.toString());
                }
            }
            IBlock ifBlock = myBlockFactory.createIfBlock(sb.toString());
            myBlockStack.push(ifBlock);
            addBlock(ifBlock);
        }
    }

    public void outAIdentifierIfThenElseStatement(AIdentifierIfThenElseStatement node) {
        defaultComparentBlockOut();
    }

    public void caseAIdentifierIfThenElseStatement(AIdentifierIfThenElseStatement node) {
        if (myMethodNow && !myTagNow) {
            inAIdentifierIfThenElseStatement(node);
            if(node.getIf() != null) {
                node.getIf().apply(this);
            }
            if(node.getLPar() != null) {
                node.getLPar().apply(this);
            }
            if(node.getIdentifier() != null) {
                node.getIdentifier().apply(this);
            }
            {
                List<PAdditionalIdentifier> copy = new ArrayList<PAdditionalIdentifier>(node.getAdditionalIdentifiers());
                for(PAdditionalIdentifier e : copy) {
                    e.apply(this);
                }
            }
            if(node.getRPar() != null) {
                node.getRPar().apply(this);
            }
            IIfBlock currentIfBlock = (IIfBlock) myCurrentBlock;
            myState = BuildSchemeState.YES_BLOCK;
            if(node.getStatementNoShortIf() != null) {
                node.getStatementNoShortIf().apply(this);
            }
            if(node.getElse() != null) {
                node.getElse().apply(this);
            }
            myCurrentBlock = currentIfBlock;
            myState = BuildSchemeState.NO_BLOCK;
            if(node.getStatement() != null) {
                node.getStatement().apply(this);
            }
            outAIdentifierIfThenElseStatement(node);
        }
    }

    //For blocks.........................................................
    
    public void inAEmptyBasicForStatement(AEmptyBasicForStatement node) {
        if (myMethodNow && !myTagNow) {
            StringBuffer sb = new StringBuffer();
            if (node.getForInit() != null) {
                sb.append(node.getForInit().toString());
                sb.append(";");
            }
            sb.append(";");
            if (node.getForUpdate() != null) {
                sb.append(node.getForUpdate().toString());
            }
            IBlock forBlock = myBlockFactory.createForBlock(sb.toString());
            myBlockStack.push(forBlock);
            addBlock(forBlock);
            myState = BuildSchemeState.FIRST_FOR_BLOCK;
        }
    }

    public void outAEmptyBasicForStatement(AEmptyBasicForStatement node) {
        defaultComparentBlockOut();
    }

    public void caseAEmptyBasicForStatement(AEmptyBasicForStatement node) {
        inAEmptyBasicForStatement(node);
        if(node.getFor() != null) {
            node.getFor().apply(this);
        }
        if(node.getLPar() != null) {
            node.getLPar().apply(this);
        }
        if(node.getForInit() != null) {
            node.getForInit().apply(this);
        }
        if(node.getSemi1() != null) {
            node.getSemi1().apply(this);
        }
        if(node.getSemi2() != null) {
            node.getSemi2().apply(this);
        }
        if(node.getForUpdate() != null) {
            //       ,
            //   .
            //node.getForUpdate().apply(this);
        }
        if(node.getRPar() != null) {
            node.getRPar().apply(this);
        }
        if(node.getStatement() != null) {
            node.getStatement().apply(this);
        }
        outAEmptyBasicForStatement(node);
    }
    
    public void inAExpressionBasicForStatement(AExpressionBasicForStatement node) {
        if (myMethodNow && !myTagNow) {
            StringBuffer sb = new StringBuffer();
            if (node.getForInit() != null) {
                sb.append(node.getForInit().toString());
                sb.append(";");
            }
            if (node.getExpression() != null) {
                sb.append(node.getExpression().toString());
                sb.append(";");
            }
            if (node.getForUpdate() != null) {
                sb.append(node.getForUpdate().toString());
            }
            IBlock forBlock = myBlockFactory.createForBlock(sb.toString());
            myBlockStack.push(forBlock);
            addBlock(forBlock);
            myState = BuildSchemeState.FIRST_FOR_BLOCK;
        }
    }

    public void outAExpressionBasicForStatement(AExpressionBasicForStatement node) {
        defaultComparentBlockOut();
    }

    public void caseAExpressionBasicForStatement(AExpressionBasicForStatement node) {
        inAExpressionBasicForStatement(node);
        if(node.getFor() != null) {
            node.getFor().apply(this);
        }
        if(node.getLPar() != null) {
            node.getLPar().apply(this);
        }
        if(node.getForInit() != null) {
            node.getForInit().apply(this);
        }
        if(node.getSemi1() != null) {
            node.getSemi1().apply(this);
        }
        if(node.getExpression() != null) {
            node.getExpression().apply(this);
        }
        if(node.getSemi2() != null) {
            node.getSemi2().apply(this);
        }
        if(node.getForUpdate() != null) {
            //       ,
            //   .
            //node.getForUpdate().apply(this);
        }
        if(node.getRPar() != null) {
            node.getRPar().apply(this);
        }
        if(node.getStatement() != null) {
            node.getStatement().apply(this);
        }
        outAExpressionBasicForStatement(node);
    }
    
    public void inAIdentifierBasicForStatement(AIdentifierBasicForStatement node) {
        if (myMethodNow && !myTagNow) {
            StringBuffer sb = new StringBuffer();
            if (node.getForInit() != null) {
                sb.append(node.getForInit().toString());
                sb.append(";");
            }
            if (node.getIdentifier() != null) {
                sb.append(node.getIdentifier().toString());
            }
            List<PAdditionalIdentifier> ai = node.getAdditionalIdentifiers();
            if (ai != null) {
                for (PAdditionalIdentifier pai: ai) {
                    sb.append(pai.toString());
                }
            }
            if (node.getForUpdate() != null) {
                sb.append(node.getForUpdate().toString());
            }
            IBlock forBlock = myBlockFactory.createForBlock(sb.toString());
            myBlockStack.push(forBlock);
            addBlock(forBlock);
            myState = BuildSchemeState.FIRST_FOR_BLOCK;
        }
    }

    public void outAIdentifierBasicForStatement(AIdentifierBasicForStatement node) {
        defaultComparentBlockOut();
    }

    public void caseAIdentifierBasicForStatement(AIdentifierBasicForStatement node) {
        inAIdentifierBasicForStatement(node);
        if(node.getFor() != null) {
            node.getFor().apply(this);
        }
        if(node.getLPar() != null) {
            node.getLPar().apply(this);
        }
        if(node.getForInit() != null) {
            node.getForInit().apply(this);
        }
        if(node.getSemi1() != null) {
            node.getSemi1().apply(this);
        }
        if(node.getIdentifier() != null) {
            node.getIdentifier().apply(this);
        }
        {
            List<PAdditionalIdentifier> copy = new ArrayList<PAdditionalIdentifier>(node.getAdditionalIdentifiers());
            for(PAdditionalIdentifier e : copy) {
                e.apply(this);
            }
        }
        if(node.getSemi2() != null) {
            node.getSemi2().apply(this);
        }
        if(node.getForUpdate() != null) {
            //        ,
            //   .
            //node.getForUpdate().apply(this);
        }
        if(node.getRPar() != null) {
            node.getRPar().apply(this);
        }
        if(node.getStatement() != null) {
            node.getStatement().apply(this);
        }
        outAIdentifierBasicForStatement(node);
    }
    
    public void inAPrimitiveExpressionEnhancedForStatement(
            APrimitiveExpressionEnhancedForStatement node) {
        if (myMethodNow && !myTagNow) {
            StringBuffer sb = new StringBuffer();
            {
                List<PModifier> copy = new ArrayList<PModifier>(node.getModifiers());
                for(PModifier e : copy) {
                    sb.append(e.toString());
                }
            }
            if(node.getPrimitiveType() != null) {
                sb.append(node.getPrimitiveType().toString());
            }
            {
                List<PDim> copy = new ArrayList<PDim>(node.getDims());
                for(PDim e : copy) {
                    sb.append(e.toString());
                }
            }
            if(node.getIdentifier() != null) {
                sb.append(node.getIdentifier().toString());
            }
            if(node.getColon() != null) {
                sb.append(node.getColon().toString());
            }
            if(node.getExpression() != null) {
                sb.append(node.getExpression().toString());
            }
            IBlock forBlock = myBlockFactory.createForBlock(sb.toString());
            myBlockStack.push(forBlock);
            addBlock(forBlock);
            myState = BuildSchemeState.FIRST_FOR_BLOCK;
        }
    }

    public void outAPrimitiveExpressionEnhancedForStatement(
            APrimitiveExpressionEnhancedForStatement node) {
        defaultComparentBlockOut();
    }
    
    public void inAPrimitiveIdentifierEnhancedForStatement(
            APrimitiveIdentifierEnhancedForStatement node) {
        if (myMethodNow && !myTagNow) {
            StringBuffer sb = new StringBuffer();
            {
                List<PModifier> copy = new ArrayList<PModifier>(node.getModifiers());
                for(PModifier e : copy) {
                    sb.append(e.toString());
                }
            }
            if(node.getPrimitiveType() != null) {
                sb.append(node.getPrimitiveType().toString());
            }
            {
                List<PDim> copy = new ArrayList<PDim>(node.getDims());
                for(PDim e : copy) {
                    sb.append(e.toString());
                }
            }
            if(node.getIdentifier1() != null) {
                sb.append(node.getIdentifier1().toString());
            }
            if(node.getColon() != null) {
                sb.append(node.getColon().toString());
            }
            if (node.getIdentifier2() != null) {
                sb.append(node.getIdentifier2().toString());
            }
            List<PAdditionalIdentifier> ai = node.getAdditionalIdentifiers();
            if (ai != null) {
                for (PAdditionalIdentifier pai: ai) {
                    sb.append(pai.toString());
                }
            }
            IBlock forBlock = myBlockFactory.createForBlock(sb.toString());
            myBlockStack.push(forBlock);
            addBlock(forBlock);
            myState = BuildSchemeState.FIRST_FOR_BLOCK;
        }
    }

    public void outAPrimitiveIdentifierEnhancedForStatement(
            APrimitiveIdentifierEnhancedForStatement node) {
        defaultComparentBlockOut();
    }

    public void inAReferenceExpressionEnhancedForStatement(
            AReferenceExpressionEnhancedForStatement node) {
        if (myMethodNow && !myTagNow) {
            StringBuffer sb = new StringBuffer();
            {
                List<PModifier> copy = new ArrayList<PModifier>(node.getModifiers());
                for(PModifier e : copy) {
                    sb.append(e.toString());
                }
            }
            if(node.getIdentifier1() != null) {
                sb.append(node.getIdentifier1().toString());
            }
            {
                List<PAdditionalIdentifier> copy = new ArrayList<PAdditionalIdentifier>(node.getAdditionalIdentifiers());
                for(PAdditionalIdentifier e : copy) {
                    sb.append(e.toString());
                }
            }
            {
                List<PTypeComponent> copy = new ArrayList<PTypeComponent>(node.getTypeComponents());
                for(PTypeComponent e : copy) {
                    sb.append(e.toString());
                }
            }
            if(node.getTypeArguments() != null) {
                sb.append(node.getTypeArguments().toString());
            }
            {
                List<PDim> copy = new ArrayList<PDim>(node.getDims());
                for(PDim e : copy) {
                    sb.append(e.toString());
                }
            }
            if(node.getIdentifier2() != null) {
                sb.append(node.getIdentifier2().toString());
            }
            if(node.getColon() != null) {
                sb.append(node.getColon().toString());
            }
            if(node.getExpression() != null) {
                sb.append(node.getExpression().toString());
            }
            IBlock forBlock = myBlockFactory.createForBlock(sb.toString());
            myBlockStack.push(forBlock);
            addBlock(forBlock);
            myState = BuildSchemeState.FIRST_FOR_BLOCK;
        }
    }

    public void outAReferenceExpressionEnhancedForStatement(
            AReferenceExpressionEnhancedForStatement node) {
        defaultComparentBlockOut();
    }
    
    public void inAReferenceIdentifierEnhancedForStatement(
            AReferenceIdentifierEnhancedForStatement node) {
        if (myMethodNow && !myTagNow) {
            StringBuffer sb = new StringBuffer();
            {
                List<PModifier> copy = new ArrayList<PModifier>(node.getModifiers());
                for(PModifier e : copy) {
                    sb.append(e.toString());
                }
            }
            if(node.getIdentifier1() != null) {
                sb.append(node.getIdentifier1().toString());
            }
            {
                List<PAdditionalIdentifier> copy = new ArrayList<PAdditionalIdentifier>(node.getAdditionalIdentifiers1());
                for(PAdditionalIdentifier e : copy) {
                    sb.append(e.toString());
                }
            }
            {
                List<PTypeComponent> copy = new ArrayList<PTypeComponent>(node.getTypeComponents());
                for(PTypeComponent e : copy) {
                    sb.append(e.toString());
                }
            }
            if(node.getTypeArguments() != null) {
                sb.append(node.getTypeArguments().toString());
            }
            {
                List<PDim> copy = new ArrayList<PDim>(node.getDims());
                for(PDim e : copy) {
                    sb.append(e.toString());
                }
            }
            if(node.getIdentifier2() != null) {
                sb.append(node.getIdentifier2().toString());
            }
            if(node.getColon() != null) {
                sb.append(node.getColon().toString());
            }
            if (node.getIdentifier3() != null) {
                sb.append(node.getIdentifier3().toString());
            }
            List<PAdditionalIdentifier> ai = node.getAdditionalIdentifiers2();
            if (ai != null) {
                for (PAdditionalIdentifier pai: ai) {
                    sb.append(pai.toString());
                }
            }
            IBlock forBlock = myBlockFactory.createForBlock(sb.toString());
            myBlockStack.push(forBlock);
            addBlock(forBlock);
            myState = BuildSchemeState.FIRST_FOR_BLOCK;
        }
    }

    public void outAReferenceIdentifierEnhancedForStatement(
            AReferenceIdentifierEnhancedForStatement node) {
        defaultComparentBlockOut();
    }

    //While blocks........................................................
    
    public void inAExpressionWhileStatement(AExpressionWhileStatement node) {
        if (myMethodNow && !myTagNow) {
            String text = "";
            if (node.getExpression() != null) {
                text = node.getExpression().toString();
            }
            IBlock whileBlock = myBlockFactory.createWhileBlock(text);
            myBlockStack.push(whileBlock);
            addBlock(whileBlock);
            myState = BuildSchemeState.FIRST_WHILE_BLOCK;
        }
    }
    
    public void outAExpressionWhileStatement(AExpressionWhileStatement node) {
        defaultComparentBlockOut();
    }
    
    public void inAIdentifierWhileStatementNoShortIf(
            AIdentifierWhileStatementNoShortIf node) {
        if (myMethodNow && !myTagNow) {
            StringBuffer sb = new StringBuffer();
            if (node.getIdentifier() != null) {
                sb.append(node.getIdentifier().toString());
            }
            List<PAdditionalIdentifier> ai = node.getAdditionalIdentifiers();
            if (ai != null) {
                for (PAdditionalIdentifier pai: ai) {
                    sb.append(pai.toString());
                }
            }
            IBlock doBlock = myBlockFactory.createDoBlock(sb.toString());
            myBlockStack.push(doBlock);
            addBlock(doBlock);
            myState = BuildSchemeState.FIRST_DO_BLOCK;
        }
    }

    public void outAIdentifierWhileStatementNoShortIf(
            AIdentifierWhileStatementNoShortIf node) {
        defaultComparentBlockOut();
    }
    
    //Do blocks.........................................................
    
    public void inAExpressionDoStatement(AExpressionDoStatement node) {
        if (myMethodNow && !myTagNow) {
            String text = "";
            if (node.getExpression() != null) {
                text = node.getExpression().toString();
            }
            IBlock doBlock = myBlockFactory.createDoBlock(text);
            myBlockStack.push(doBlock);
            addBlock(doBlock);
            myState = BuildSchemeState.FIRST_DO_BLOCK;
        }
    }
    
    public void outAExpressionDoStatement(AExpressionDoStatement node) {
        defaultComparentBlockOut();
    }
    
    public void inAIdentifierDoStatement(AIdentifierDoStatement node) {
        if (myMethodNow && !myTagNow) {
            StringBuffer sb = new StringBuffer();
            if (node.getIdentifier() != null) {
                sb.append(node.getIdentifier().toString());
            }
            List<PAdditionalIdentifier> ai = node.getAdditionalIdentifiers();
            if (ai != null) {
                for (PAdditionalIdentifier pai: ai) {
                    sb.append(pai.toString());
                }
            }
            IBlock doBlock = myBlockFactory.createDoBlock(sb.toString());
            myBlockStack.push(doBlock);
            addBlock(doBlock);
            myState = BuildSchemeState.FIRST_DO_BLOCK;
        }
    }

    public void outAIdentifierDoStatement(AIdentifierDoStatement node) {
        defaultComparentBlockOut();
    }
    
    //Switch blocks..........................................................
    
    public void inAExpressionSwitchStatement(AExpressionSwitchStatement node) {
        if (myMethodNow && !myTagNow) {
            String text = "";
            if (node.getExpression() != null) {
                text = node.getExpression().toString();
            }
            IBlock switchBlock = myBlockFactory.createSwitchBlock(text);
            myBlockStack.push(switchBlock);
            addBlock(switchBlock);
        }
    }

    public void outAExpressionSwitchStatement(AExpressionSwitchStatement node) {
        defaultComparentBlockOut();
    }
    
    public void inAIdentifierSwitchStatement(AIdentifierSwitchStatement node) {
        if (myMethodNow && !myTagNow) {
            StringBuffer sb = new StringBuffer();
            if (node.getIdentifier() != null) {
                sb.append(node.getIdentifier().toString());
            }
            List<PAdditionalIdentifier> ai = node.getAdditionalIdentifiers();
            if (ai != null) {
                for (PAdditionalIdentifier pai: ai) {
                    sb.append(pai.toString());
                }
            }
            IBlock switchBlock = myBlockFactory.createSwitchBlock(sb.toString());
            myBlockStack.push(switchBlock);
            addBlock(switchBlock);
        }
    }

    public void outAIdentifierSwitchStatement(AIdentifierSwitchStatement node) {
        defaultComparentBlockOut();
    }

    public void caseASwitchBlockStatementGroup(ASwitchBlockStatementGroup node) {
        inASwitchBlockStatementGroup(node);
        {
            List<PSwitchLabel> copy = new ArrayList<PSwitchLabel>(node.getSwitchLabels());
            for(PSwitchLabel e : copy) {
                e.apply(this);
            }
        }
        {
            List<PBlockStatement> copy = new ArrayList<PBlockStatement>(node.getBlockStatements());
            IBlock currentSB = myCurrentBlock;
            myState = BuildSchemeState.SWITCH_BLOCK;
            for(PBlockStatement e : copy) {
                e.apply(this);
            }
            myCurrentBlock = currentSB;
            myState = BuildSchemeState.BEGIN_STATE;
        }
        outASwitchBlockStatementGroup(node);
    }

    public void inAExpressionSwitchLabel(AExpressionSwitchLabel node) {
        if (myMethodNow && !myTagNow) {
            if (node.getConstantExpression() != null) {
                ((ISwitchBlock) myCurrentBlock).addCase(
                        node.getConstantExpression().toString(), null); 
            }
        }

    }

    public void inAIdentifierSwitchLabel(AIdentifierSwitchLabel node) {
        if (myMethodNow && !myTagNow) {
            StringBuffer sb = new StringBuffer();
            if (node.getIdentifier() != null) {
                sb.append(node.getIdentifier().toString());
            }
            List<PAdditionalIdentifier> ai = node.getAdditionalIdentifiers();
            if (ai != null) {
                for (PAdditionalIdentifier pai: ai) {
                    sb.append(pai.toString());
                }
            }
            ((ISwitchBlock) myCurrentBlock).addCase(sb.toString(), null); 
        }
    }
    
    public void inADefaultSwitchLabel(ADefaultSwitchLabel node) {
        if (myMethodNow && !myTagNow) {
            if (node.getDefault() != null) {
                ((ISwitchBlock) myCurrentBlock).addCase(
                        node.getDefault().toString(), null); 
            }
        }
    }
    
    //Try blocks............................................................
    
    public void inACatchTryStatement(ACatchTryStatement node) {
        if (myMethodNow && !myTagNow) {
            ITryBlock tb = myBlockFactory.createTryBlock();
            myBlockStack.push(tb);
            addBlock(tb);
            myState = BuildSchemeState.TRY_BLOCK;
        }
    }

    public void outACatchTryStatement(ACatchTryStatement node) {
        defaultComparentBlockOut();
    }

    public void caseACatchTryStatement(ACatchTryStatement node) {
        inACatchTryStatement(node);
        if(node.getTry() != null) {
            node.getTry().apply(this);
        }
        if(node.getBlock() != null) {
            node.getBlock().apply(this);
        }
        {
            if (myMethodNow && !myTagNow) {
                myCurrentBlock = myBlockStack.peek();
            }
            List<PCatchClause> copy = new ArrayList<PCatchClause>(node.getCatchClauses());
            for(PCatchClause e : copy) {
                e.apply(this);
            }
        }
        outACatchTryStatement(node);
    }
   
    public void inAFinallyTryStatement(AFinallyTryStatement node) {
        if (myMethodNow && !myTagNow) {
            ITryBlock tb = myBlockFactory.createTryBlock();
            myBlockStack.push(tb);
            addBlock(tb);
            myState = BuildSchemeState.TRY_BLOCK;
        }
    }

    public void outAFinallyTryStatement(AFinallyTryStatement node) {
        defaultComparentBlockOut();
    }
    
    public void caseAFinallyTryStatement(AFinallyTryStatement node) {
        inAFinallyTryStatement(node);
        if(node.getTry() != null) {
            node.getTry().apply(this);
        }
        if(node.getBlock() != null) {
            node.getBlock().apply(this);
        }
        {
            if (myMethodNow && !myTagNow) {
                myCurrentBlock = myBlockStack.peek();
            }
            List<PCatchClause> copy = new ArrayList<PCatchClause>(node.getCatchClauses());
            for(PCatchClause e : copy) {
                e.apply(this);
            }
        }
        if(node.getFinally() != null)
        {
            node.getFinally().apply(this);
        }
        outAFinallyTryStatement(node);
    }

    public void inACatchClause(ACatchClause node) {
        if (myMethodNow && !myTagNow) {
            ITryBlock tb = (ITryBlock) myCurrentBlock;
            IBlock catchBlock = myBlockFactory.createActionBlock(
                    node.getFormalParameter().toString());
            tb.addCatch(catchBlock);
            myCurrentBlock = catchBlock;
            myState = BuildSchemeState.BEGIN_STATE;
        }
    }

    public void outACatchClause(ACatchClause node) {
        if (myMethodNow && !myTagNow) {
            myCurrentBlock = myBlockStack.peek();
        }
    }

    public void inAFinally(AFinally node) {
        if (myMethodNow && !myTagNow) {
            myState = BuildSchemeState.FINALLY;
        }
    }
    
    //Tags
    
    public void inABeginTag(ABeginTag node) {
        if (myMethodNow) {
            String tag = node.getBegin().toString();
            addActionBlock(tag.substring(6));
            myTagNow = true;
        }
    }
    
    public void inAEndTag(AEndTag node) {
        if (myMethodNow) {
            myTagNow = false;
        }
    }
    
    //Other blocks..........................................................
    
    public void inABreakStatement(ABreakStatement node) {
        if (myMethodNow && !myTagNow) {
            IBlock block;
            if (node.getIdentifier() == null) {
                block = myBlockFactory.createEndBlock("break");
            } else {
                block = myBlockFactory.createEndBlock("break" 
                        + node.getIdentifier().toString());
            }
            addBlock(block);
        }
    }
    
}
