package org.amse.ak.schemebuilder.builder;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PushbackReader;

import javax.swing.tree.DefaultMutableTreeNode;

import org.sablecc.java.lexer.Lexer;
import org.sablecc.java.lexer.LexerException;
import org.sablecc.java.node.Node;
import org.sablecc.java.parser.Parser;
import org.sablecc.java.parser.ParserException;

public class ChooseMethodBuilder {
    
    private final File myFile;
    
    public ChooseMethodBuilder(File f) {
        myFile = f;
    }
    
    public DefaultMutableTreeNode build() throws SchemeBuilderException {
        try {
            DefaultMutableTreeNode root  = new DefaultMutableTreeNode(myFile.getName());
            Lexer lexer = new Lexer(new PushbackReader(new BufferedReader(
                    new FileReader(myFile)), 1024));
            Parser parser = new Parser(lexer);
            Node ast = parser.parse();
            FindMethodVisitor walker = new FindMethodVisitor(root);
            ast.apply(walker);
            if (root.getChildCount() == 0) {
                throw new SchemeBuilderException("No methods in this class.");
            }
            return root;
        } catch (FileNotFoundException e) {
            throw new SchemeBuilderException("Can't open file.", e);
        } catch (IOException e) {
            throw new SchemeBuilderException("Can't open file.", e);
        } catch (LexerException e) {
            throw new SchemeBuilderException("Can't parse file.", e);
        } catch (ParserException e) {
            throw new SchemeBuilderException("Can't parse file.", e);
        }
    }

}
