package org.amse.ak.schemebuilder.builder;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PushbackReader;

import org.sablecc.java.lexer.Lexer;
import org.sablecc.java.lexer.LexerException;
import org.sablecc.java.node.Node;
import org.sablecc.java.parser.Parser;
import org.sablecc.java.parser.ParserException;

import org.amse.ak.schemebuilder.model.IMethod;
import org.amse.ak.schemebuilder.model.IScheme;
import org.amse.ak.schemebuilder.model.impl.Scheme;

public class SchemeBuilder implements ISchemeBuilder {

    private final IScheme myScheme;
    
    public SchemeBuilder(IMethod method) {
        myScheme = new Scheme(method);
    }

    public IScheme build() throws SchemeBuilderException {
        try {
            Lexer lexer = new Lexer(new PushbackReader(new BufferedReader(
                    new FileReader(myScheme.getMethod().getClassPath())), 1024));
            Parser parser = new Parser(lexer);
            Node ast = parser.parse();
            
            BuildSchemeVisitor visitor = new BuildSchemeVisitor(myScheme);
            ast.apply(visitor);
            if (visitor.isMethodExist()) {
                return myScheme;
            } else {
                throw new SchemeBuilderException("Can't find method in class.");
            }
        } catch (FileNotFoundException e) {
            throw new SchemeBuilderException("Can't open file.", e);
        } catch (IOException e) {
            throw new SchemeBuilderException("Can't open file.", e);
        } catch (LexerException e) {
            throw new SchemeBuilderException("Can't parse file.", e);
        } catch (ParserException e) {
            throw new SchemeBuilderException("Can't parse file.", e);
        }
    }

}
