package org.amse.ak.schemebuilder.io.writer;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.amse.ak.schemebuilder.io.SchemeIOInterface;
import org.amse.ak.schemebuilder.model.IMethod;
import org.amse.ak.schemebuilder.model.IScheme;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class SchemeWriter implements SchemeIOInterface {
    
    private IScheme myScheme;
    
    private Node myDocument;
      
    public SchemeWriter(IScheme scheme) throws ParserConfigurationException {
        myScheme = scheme;
        myDocument = createDocument();
    }
      
    private Node createDocument() throws ParserConfigurationException {
        Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        Element scheme = doc.createElement(SCHEME);
        addMethod(scheme, doc);
        WriteVisitor visitor = new WriteVisitor(doc, scheme);
        myScheme.getBeginBlock().apply(visitor);
        return scheme;
    }
      
    private void addMethod(Element scheme, Document doc) {
        Element me = doc.createElement(METHOD);
        IMethod method = myScheme.getMethod();
        me.setAttribute(PATH, method.getClassPath());
        me.setAttribute(NAME, method.getName());
        scheme.appendChild(me);
    }
    
    public void write() throws IOException, TransformerConfigurationException,
        TransformerException {
        TransformerFactory tf = TransformerFactory.newInstance();
        tf.setAttribute("indent-number", new Integer(2));
        Transformer tr = tf.newTransformer();
        tr.setOutputProperty(OutputKeys.INDENT, "yes");
        OutputStream out = new FileOutputStream(myScheme.getFile());
        StreamResult res = new StreamResult(new OutputStreamWriter(out, "utf-8"));
        tr.transform(new DOMSource(myDocument), res);
        OutputStream os = res.getOutputStream();
        if (os != null) {
            os.close();
        }
    }

}
