package org.amse.ak.schemebuilder.io.writer;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.Set;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.amse.ak.schemebuilder.io.SettingsIOInterface;
import org.amse.ak.schemebuilder.view.settings.ViewSettings;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class SettingsWriter implements SettingsIOInterface {
    
    private final ViewSettings mySettings;
    
    private final Node myDocument;
      
    public SettingsWriter(ViewSettings settings) throws ParserConfigurationException {
        mySettings = settings;
        myDocument = createDocument();
    }
    
    private Node createDocument() throws ParserConfigurationException {
        Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        Element settings = doc.createElement(SETTINGS);
        Set<String> properties = mySettings.getAllProperties();
        for(String property: properties) {
            Element pe = doc.createElement(property);
            pe.setAttribute(VALUE, mySettings.getProperty(property));
            settings.appendChild(pe);
        }
        return settings;
    }
    
    public void write() throws IOException, TransformerConfigurationException,
            TransformerException {
        TransformerFactory tf = TransformerFactory.newInstance();
        tf.setAttribute("indent-number", new Integer(2));
        Transformer tr = tf.newTransformer();
        tr.setOutputProperty(OutputKeys.INDENT, "yes");
        OutputStream out = new FileOutputStream(SETTINGS_FILE);
        StreamResult res = new StreamResult(new OutputStreamWriter(out, "utf-8"));
        tr.transform(new DOMSource(myDocument), res);
        OutputStream os = res.getOutputStream();
        if (os != null) {
            os.close();
        }
    }

}
