package org.amse.ak.schemebuilder.model.impl;

import java.io.File;

import org.amse.ak.schemebuilder.model.IDoBlock;
import org.amse.ak.schemebuilder.model.IInvocationBlock;
import org.amse.ak.schemebuilder.model.ITryBlock;
import org.amse.ak.schemebuilder.model.IWhileBlock;
import org.amse.ak.schemebuilder.model.ISchemeVisitor;
import org.amse.ak.schemebuilder.model.IActionBlock;
import org.amse.ak.schemebuilder.model.IBeginBlock;
import org.amse.ak.schemebuilder.model.IEndBlock;
import org.amse.ak.schemebuilder.model.IForBlock;
import org.amse.ak.schemebuilder.model.IIfBlock;
import org.amse.ak.schemebuilder.model.IMethod;
import org.amse.ak.schemebuilder.model.IScheme;
import org.amse.ak.schemebuilder.model.ISwitchBlock;

public class Scheme implements IScheme {

    private final IMethod myMethod;
    
    private final IBeginBlock myBeginBlock;

    private File myFile = null;
    
    public Scheme(IMethod method) {
        myMethod = method;
        myBeginBlock = new BeginBlock(method.getName());
    }

    public Scheme(IMethod method, String beginBlockText) {
        myMethod = method;
        myBeginBlock = new BeginBlock(beginBlockText);
    }
    
    public IMethod getMethod() {
        return myMethod;
    }
    
    public IBeginBlock getBeginBlock() {
        return myBeginBlock;
    }

    public File getFile() {
        return myFile;
    }
    
    public void setFile(File f) {
        myFile = f;
    }
    
    public String toString() {
        final StringBuffer sb = new StringBuffer();
        if (myBeginBlock != null) {
            myBeginBlock.apply(new ISchemeVisitor() {

                public void caseActionBlock(IActionBlock actionBlock) {
                    if (actionBlock != null) {
                        sb.append(actionBlock);
                        if (actionBlock.getNext() != null) {
                            sb.append(" -> ");
                            actionBlock.getNext().apply(this);
                        }
                    }
                }

                public void caseBeginBlock(IBeginBlock beginBlock) {
                    if (beginBlock != null) {
                        sb.append(beginBlock);
                        if (beginBlock.getNext() != null) {
                            sb.append(" -> ");
                            beginBlock.getNext().apply(this);
                        }
                    }
                }

                public void caseDoBlock(IDoBlock doWhileBlock) {
                    if (doWhileBlock != null) {
                        sb.append("{");
                        if (doWhileBlock.getFirstBlock() != null) {
                            doWhileBlock.getFirstBlock().apply(this);
                        }
                        sb.append("} ");
                        sb.append(doWhileBlock);
                        if (doWhileBlock.getNext() != null) {
                            sb.append(" -> ");
                            doWhileBlock.getNext().apply(this);
                        }
                    }
                }
                
                public void caseEndBlock(IEndBlock endBlock) {
                    if (endBlock != null) {
                        sb.append(endBlock);
                    }
                }

                public void caseForBlock(IForBlock forBlock) {
                    if (forBlock != null) {
                        sb.append(forBlock);
                        sb.append(" {");
                        if (forBlock.getFirstBlock() != null) {
                            forBlock.getFirstBlock().apply(this);
                        }
                        sb.append("}");
                        if (forBlock.getNext() != null) {
                            sb.append(" -> ");
                            forBlock.getNext().apply(this);
                        }
                    }
                }

                public void caseIfBlock(IIfBlock ifBlock) {
                    if (ifBlock != null) {
                        sb.append(ifBlock);
                        sb.append(" {");
                        if (ifBlock.getYesBlock() != null) {
                            ifBlock.getYesBlock().apply(this);
                        }
                        sb.append("} else {");
                        if (ifBlock.getNoBlock() != null) {
                            ifBlock.getNoBlock().apply(this);
                        }
                        sb.append("}");
                        if (ifBlock.getNext() != null) {
                            sb.append(" -> ");
                            ifBlock.getNext().apply(this);
                        }
                    }
                }
                
                public void caseInvocationBlock(IInvocationBlock invocationBlock) {
                    if (invocationBlock != null) {
                        sb.append(invocationBlock);
                        if (invocationBlock.getNext() != null) {
                            sb.append(" -> ");
                            invocationBlock.getNext().apply(this);
                        }
                    }
                }

                public void caseSwitchBlock(ISwitchBlock switchBlock) {
                    if (switchBlock != null) {
                        sb.append(switchBlock);
                        if (switchBlock.getNext() != null) {
                            sb.append(" -> ");
                            switchBlock.getNext().apply(this);
                        }
                    }
                }
                
                public void caseTryBlock(ITryBlock tryBlock) {
                    if (tryBlock != null) {
                        sb.append(tryBlock);
                        if (tryBlock.getNext() != null) {
                            sb.append(" -> ");
                            tryBlock.getNext().apply(this);
                        }
                    }
                }
                
                public void caseWhileBlock(IWhileBlock whileBlock) {
                    if (whileBlock != null) {
                        sb.append(whileBlock);
                        sb.append(" {");
                        if (whileBlock.getFirstBlock() != null) {
                            whileBlock.getFirstBlock().apply(this);
                        }
                        sb.append("}");
                        if (whileBlock.getNext() != null) {
                            sb.append(" -> ");
                            whileBlock.getNext().apply(this);
                        }
                    }
                }
                
            });
        }
        return sb.toString();
    }

}
