package org.amse.ak.schemebuilder.view;

import java.awt.Font;
import java.awt.Point;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import javax.swing.JPanel;

import org.amse.ak.schemebuilder.model.*;
import org.amse.ak.schemebuilder.model.impl.BlockFactory;
import org.amse.ak.schemebuilder.view.drawer.*;
import org.amse.ak.schemebuilder.view.settings.ViewSettings;

class LayoutVisitor implements ISchemeVisitor {
    
    private final JPanel mySchemePanel;
    
    private final ViewSettings mySettings;

    
    //  .
    private int myDepth;
    
    //  .
    private int myWidth;
    
    //    .
    private int myCurrentBlockWidth = 0;
    
    //    .
    private int myCurrentBlockHeight = 0;
    
    // ,        IEndBlock.
    private boolean myEndBlockNow = false;
    
    // ,         Try .
    private boolean myNoNeedArrow = false;
    
    private Stack<WidthInformation> myNestingBlockStack = new Stack<WidthInformation>();
    
    private final int myCenter = ViewSettings.CENTER;

    //    
    private final int myHorizontalStandoff = ViewSettings.HORIZONTAL_STANDOFF_IN_BLOCK
                                             + ViewSettings.BORDER;
    
    private final int myVerticalStandoff = ViewSettings.VERTICAL_STANDOFF_IN_BLOCK
                                             + ViewSettings.BORDER;
    
    //     
    private final int myNoArrowStandoff = ViewSettings.NO_ARROW_STANDOFF;
    
    private final int myYesArrowStandoff = ViewSettings.YES_ARROW_STANDOFF;
    
    //           .
    private final int myBackArrowStandoff = ViewSettings.BACK_ARROW_STANDOFF;
    
    //        .
    private final int myNextArrowStandoff = ViewSettings.NEXT_ARROW_STANDOFF;
    
    private final FontRenderContext myFontRenderContext = ViewSettings.FONT_RENDER_CONTEXT;
    
    private final int myPointSize = ViewSettings.POINT_SIZE;
    
    public LayoutVisitor(JPanel schemePanel, ViewSettings settings, int depth, int width) {
        mySchemePanel = schemePanel;
        mySettings = settings;
        myDepth = depth;
        myWidth = width;
        myNestingBlockStack.push(new WidthInformation());
    }
    
    public int getWidth() {
        return myWidth;
    }
    
    public int getDepth() {
        return myDepth;
    }
    
    private void calcInnerBlockSize() {
        WidthInformation info = myNestingBlockStack.pop();
        WidthInformation parentInfo  = myNestingBlockStack.peek();
        if (info.getRightWidth() > parentInfo.getRightWidth()) {
            parentInfo.setRightWidth(info.getRightWidth());
        }
        if (info.getLeftWidth() > parentInfo.getLeftWidth()) {
            parentInfo.setLeftWidth(info.getLeftWidth());
        }
    }
    
    private void addBlockDrawer(BlockDrawer bd) {
        int width = myCurrentBlockWidth;
        int height = myCurrentBlockHeight;
        //   BlockDrawer'.
        //  -    + 2  (  ).
        //  -    + 2  (  ).
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        bd.setBounds(myCenter - width / 2 - myHorizontalStandoff - lineWidth / 2, 
                myDepth, width + 2 * myHorizontalStandoff + lineWidth,
                height +  2 * myVerticalStandoff + lineWidth);
        //      .
        if (width / 2 + myHorizontalStandoff > myWidth) {
            myWidth = width / 2 + myHorizontalStandoff;
        }
        mySchemePanel.add(bd);
 
        //        ,
        //    
        myDepth += height + 2 * myVerticalStandoff;
        WidthInformation info = myNestingBlockStack.peek();
        if (myCurrentBlockWidth / 2 > info.getRightWidth()) {
            info.setRightWidth(myCurrentBlockWidth / 2);
        }
        if (myCurrentBlockWidth / 2 > info.getLeftWidth()) {
            info.setLeftWidth(myCurrentBlockWidth / 2);
        }
    }

    //     .
    private void addArrowDrawer() {
        ArrowDrawer ad = new ArrowDrawer(mySettings);
        ad.setBounds(myCenter - ViewSettings.ARROW_WEDTH / 2,
                myDepth, ViewSettings.ARROW_WEDTH,
                ViewSettings.ARROW_LENGTH);
        myDepth += ViewSettings.ARROW_LENGTH;
        mySchemePanel.add(ad);
    }
    
    //   .
    private void addPointDrawer() {
        PointDrawer pd1 = new PointDrawer(mySettings);
        pd1.setBounds(myCenter - ViewSettings.POINT_SIZE / 2,
                myDepth, ViewSettings.POINT_SIZE, ViewSettings.POINT_SIZE);
        myDepth += ViewSettings.POINT_SIZE;
        mySchemePanel.add(pd1);
    }

    //        
    //      .
    private List<String> correctText(IBlock b) {
        String text = b.getText();
        int size = Integer.parseInt(mySettings.getProperty("fontSize"));
        int style = Integer.parseInt(mySettings.getProperty("fontStyle"));
        String name = mySettings.getProperty("fontName");
        Font font = new Font(name, style, size);
        Rectangle2D textBounds = font.getStringBounds(text,
                myFontRenderContext);
        int textWidth = (int) Math.round(textBounds.getWidth());
        int maxBlockWidth = Integer.parseInt(mySettings.getProperty("maxBlockWidth"));
        int lines = textWidth / maxBlockWidth;
        List<String> res = new ArrayList<String>();
        if (lines == 0) {
            res.add(text);
            myCurrentBlockWidth = (int) Math.round(textBounds.getWidth());
            myCurrentBlockHeight = (int) Math.round(textBounds.getHeight());
            return res;
        }
        int length = text.length();
        int width = 0;
        int height = 0;
        for (int i = 0; i < lines; i++) {
            String line = text.substring(i * length / lines,  (i + 1) * length / lines);
            Rectangle2D lineBounds = font.getStringBounds(line, myFontRenderContext);
            height += (int) Math.round(lineBounds.getHeight());
            int lineWidth = (int) Math.round(lineBounds.getWidth());
            if (lineWidth > width) {
                width = lineWidth;
            }
            res.add(line);
        }
        myCurrentBlockWidth = width;
        myCurrentBlockHeight = height;
        return res;
    }
    
    public void caseActionBlock(IActionBlock actionBlock) {
        if (actionBlock != null) {
            addBlockDrawer(new ActionBlockDrawer(correctText(actionBlock),
                    mySettings));
            if (actionBlock.getNext() != null) {
                addArrowDrawer();
                actionBlock.getNext().apply(this);
            }
        }
    }

    public void caseBeginBlock(IBeginBlock beginBlock) {
        if (beginBlock != null) {
            addBlockDrawer(new BeginEndBlockDrawer(correctText(beginBlock),
                    mySettings));
            if (beginBlock.getNext() != null) {
                addArrowDrawer();
                beginBlock.getNext().apply(this);
            }
        }
    }
    
    private void addDoBackArrow(int width, int height, int pointDepth) {
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        WidthInformation info = myNestingBlockStack.peek();
        int extraWidth = info.getLeftWidth();
        int ifArrowCenter = myPointSize + ViewSettings.IF_ARROW_X + extraWidth + lineWidth;
        List<Point> breaks = new ArrayList<Point>();
        breaks.add(new Point(ifArrowCenter - width / 2 - myHorizontalStandoff,
                myDepth - pointDepth + height / 2 + myVerticalStandoff));
        breaks.add(new Point(lineWidth, myDepth - pointDepth + height / 2 + myVerticalStandoff));
        breaks.add(new Point(lineWidth,  myPointSize / 2));
        breaks.add(new Point(ifArrowCenter - myPointSize / 2, myPointSize / 2));
        BrokenArrowDrawer ifad = new BrokenArrowDrawer(breaks,
                new Point(lineWidth, myDepth - pointDepth + height / 2 + myVerticalStandoff),
                BrokenArrowDrawer.YES,
                BrokenArrowDrawer.HORIZONTAL, false, mySettings);
        ifad.setBounds(myCenter - ifArrowCenter,
                pointDepth, ifArrowCenter + lineWidth,
                myDepth - pointDepth + height / 2 + myVerticalStandoff + lineWidth);
        myNestingBlockStack.peek().setLeftWidth(ifArrowCenter);
        mySchemePanel.add(ifad);
        info.setLeftWidth(ifArrowCenter);
    }
    
    //         .
    private void addDoContinuedArrow(int width, int height, int whileBlockDepth, boolean isSimpleLine) {
        int ifArrowCenter = width / 2 + myHorizontalStandoff
                + ViewSettings.IF_ARROW_X;// + extraWidth;
        List<Point> breaks = new ArrayList<Point>();
        breaks.add(new Point(width / 2 + myHorizontalStandoff 
                + ViewSettings.ARROW_WEDTH / 2,
                height / 2 + myVerticalStandoff));
        breaks.add(new Point(ifArrowCenter + ViewSettings.ARROW_WEDTH / 2,
                height / 2 + myVerticalStandoff));
        breaks.add(new Point(ifArrowCenter + ViewSettings.ARROW_WEDTH / 2,
                height + 2 * myVerticalStandoff + myYesArrowStandoff));
        breaks.add(new Point(ViewSettings.ARROW_WEDTH / 2,
                height + 2 * myVerticalStandoff + myYesArrowStandoff));
        breaks.add(new Point(ViewSettings.ARROW_WEDTH / 2,
                height + 2 * myVerticalStandoff + myYesArrowStandoff
                       + ViewSettings.ARROW_LENGTH));
        BrokenArrowDrawer ifad = new BrokenArrowDrawer(
                breaks, new Point(width / 2 + myHorizontalStandoff 
                        + ViewSettings.ARROW_WEDTH / 2,
                        height / 2 + myVerticalStandoff),
                BrokenArrowDrawer.NO,
                BrokenArrowDrawer.VERTICAL, isSimpleLine, mySettings);
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        ifad.setBounds(myCenter - ViewSettings.ARROW_WEDTH / 2,
                whileBlockDepth, ViewSettings.ARROW_WEDTH / 2 
                + ifArrowCenter + lineWidth,
                height + 2 * myVerticalStandoff + myYesArrowStandoff
                + ViewSettings.ARROW_LENGTH);
        myDepth += myYesArrowStandoff + ViewSettings.ARROW_LENGTH;
        if (ifArrowCenter + lineWidth > myNestingBlockStack.peek().getRightWidth()) {
            myNestingBlockStack.peek().setRightWidth(ifArrowCenter + lineWidth);
        }
        mySchemePanel.add(ifad);
    }

    public void caseDoBlock(IDoBlock doBlock) {
        if (doBlock != null) {
            addPointDrawer();
            int pointDepth = myDepth - myPointSize;                       
            addArrowDrawer();
            
            WidthInformation info = new WidthInformation();
            myNestingBlockStack.push(info);
            
            //    
            if (doBlock.getFirstBlock() != null) {
                //  .
                doBlock.getFirstBlock().apply(this);
                //    
                addArrowDrawer();
            }
            
            IfBlockDrawer ifbd = new IfBlockDrawer(correctText(doBlock),
                    mySettings);
            //   IFBlockDrawer'.
            //  -     + 2  (  ).
            //  -     + 2  (  ).
            ifbd.setBounds(myCenter - myCurrentBlockWidth - myHorizontalStandoff,
                    myDepth, 2 * (myCurrentBlockWidth + myHorizontalStandoff),
                    2 * (myCurrentBlockHeight + myVerticalStandoff));
            //      .
            if (myCurrentBlockWidth + myHorizontalStandoff > myWidth) {
                myWidth = myCurrentBlockWidth + myHorizontalStandoff;
            }
            mySchemePanel.add(ifbd);
            if (myCurrentBlockWidth + myHorizontalStandoff > info.getLeftWidth()) {
                info.setLeftWidth(myCurrentBlockWidth + myHorizontalStandoff);
            }
            
            int width = 2 * myCurrentBlockWidth;
            int height = 2 * myCurrentBlockHeight;
            int whileBlockDepth = myDepth;
            
            addDoBackArrow(width, height, pointDepth);
            myDepth += height + 2 * myVerticalStandoff;
            calcInnerBlockSize();
            //      :
            if (doBlock.getNext() != null) {
                // ,    
                addDoContinuedArrow(width, height, whileBlockDepth, false);
                //  .
                doBlock.getNext().apply(this);
            } else {
                // ,      (   Try),
                //     .
                if (myNestingBlockStack.size() == 2) {
                    if (!myNoNeedArrow) {
                        addDoContinuedArrow(width, height, whileBlockDepth, true);
                    }
                    myNoNeedArrow = false;
                } else if (myNestingBlockStack.size() > 2) {
                    addDoContinuedArrow(width, height, whileBlockDepth, true);
                }
            }
        }
    }
    
    public void caseEndBlock(IEndBlock endBlock) {
        if (endBlock != null) {
            addBlockDrawer(new BeginEndBlockDrawer(correctText(endBlock),
                    mySettings));
            myEndBlockNow = true;
        }
    }

    public void caseForBlockIn(IForBlock forBlock) {
        ForBlockDrawer fbd = new ForBlockDrawer(correctText(forBlock),
                mySettings);
        int breast = ViewSettings.FOR_BREAST + myHorizontalStandoff;
        WidthInformation info = new WidthInformation(myCurrentBlockWidth / 2 + breast,
                myCurrentBlockWidth / 2 + breast);
        myNestingBlockStack.push(info);
        //   ForBlockDrawer'.
        //  -   + 2 .  (  ).
        //  -    + 2  (  ).
        fbd.setBounds(myCenter - myCurrentBlockWidth / 2 - breast,
                myDepth, myCurrentBlockWidth + 2 * breast,
                myCurrentBlockHeight + 2 * myVerticalStandoff);
        //      .
        if (myCurrentBlockWidth / 2 + breast > myWidth) {
            myWidth = myCurrentBlockWidth / 2 + breast;
        }
        mySchemePanel.add(fbd);
    }
    
    //          .
    public void addBackArrow(int width, int height, int forBlockDepth) {
        WidthInformation info = myNestingBlockStack.peek();
        int extraWidth = 0;
        if (info.getLeftWidth() > width / 2) {
            extraWidth = info.getLeftWidth() - width / 2;
        }
        int breast = ViewSettings.FOR_BREAST + myHorizontalStandoff;
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        int ifArrowCenter = width / 2 + breast 
                + ViewSettings.IF_ARROW_X + extraWidth + lineWidth;
        List<Point> breaks = new ArrayList<Point>();
        breaks.add(new Point(ifArrowCenter, myDepth - forBlockDepth));
        breaks.add(new Point(ifArrowCenter, myDepth - forBlockDepth 
                + myBackArrowStandoff));
        breaks.add(new Point(lineWidth,  myDepth - forBlockDepth 
                + myBackArrowStandoff));
        breaks.add(new Point(lineWidth,  height / 2 + myVerticalStandoff));
        breaks.add(new Point(ifArrowCenter - width / 2 - breast,
                height / 2 + myVerticalStandoff));
        BrokenArrowDrawer ifad = new BrokenArrowDrawer(breaks, null, null,
                BrokenArrowDrawer.HORIZONTAL, false, mySettings);
        ifad.setBounds(myCenter - ifArrowCenter,
                forBlockDepth, ifArrowCenter + lineWidth,
                myDepth - forBlockDepth + myBackArrowStandoff + lineWidth);
        myNestingBlockStack.peek().setLeftWidth(ifArrowCenter);
        mySchemePanel.add(ifad);
        info.setLeftWidth(ifArrowCenter);
    }
    
    //    ,    .
    public void addArrowToNextBlock(int width, int height, int forBlockDepth, boolean isSimpleLine) {
        WidthInformation info = myNestingBlockStack.peek();
        int extraWidth = 0;
        if (info.getRightWidth() > width / 2) {
            extraWidth = info.getRightWidth() - width / 2;
        }
        int breast = ViewSettings.FOR_BREAST + myHorizontalStandoff;
        int ifArrowCenter = width / 2 + breast
                + ViewSettings.IF_ARROW_X + extraWidth;
        List<Point> breaks = new ArrayList<Point>();
        breaks.add(new Point(ViewSettings.ARROW_WEDTH / 2
                + ifArrowCenter - ViewSettings.IF_ARROW_X - extraWidth,
                height / 2 + myVerticalStandoff));
        breaks.add(new Point(ViewSettings.ARROW_WEDTH / 2
                + ifArrowCenter,
                height / 2 + myVerticalStandoff));
        breaks.add(new Point(ViewSettings.ARROW_WEDTH / 2
                + ifArrowCenter,
                myDepth - forBlockDepth + myNextArrowStandoff));
        breaks.add(new Point(ViewSettings.ARROW_WEDTH / 2,
                myDepth - forBlockDepth + myNextArrowStandoff));
        breaks.add(new Point(ViewSettings.ARROW_WEDTH / 2,
                myDepth - forBlockDepth + myNextArrowStandoff
                        + ViewSettings.ARROW_LENGTH));
        BrokenArrowDrawer ifad = new BrokenArrowDrawer(breaks,
                new Point(ViewSettings.ARROW_WEDTH / 2
                        + ifArrowCenter
                        - ViewSettings.IF_ARROW_X - extraWidth,
                        height / 2 + myVerticalStandoff),
                null, BrokenArrowDrawer.VERTICAL, isSimpleLine, mySettings);
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        ifad.setBounds(myCenter - ViewSettings.ARROW_WEDTH / 2,
                forBlockDepth, ViewSettings.ARROW_WEDTH / 2 + ifArrowCenter + lineWidth,
                myDepth - forBlockDepth + myNextArrowStandoff + ViewSettings.ARROW_LENGTH);
        myDepth += myNextArrowStandoff + ViewSettings.ARROW_LENGTH;
        mySchemePanel.add(ifad);
        if (ifArrowCenter + lineWidth > info.getRightWidth()) {
            info.setRightWidth(ifArrowCenter + lineWidth);
        }
    }
          
    public void caseForBlock(IForBlock forBlock) {
        if (forBlock != null) {
            caseForBlockIn(forBlock);
            int width = myCurrentBlockWidth;
            int height = myCurrentBlockHeight;
            int forBlockDepth = myDepth;
            myDepth += height + 2 * myVerticalStandoff;
            //    
            if (forBlock.getFirstBlock() != null) {
                // ,     
                addArrowDrawer();
                //   .
                forBlock.getFirstBlock().apply(this);
                //  
                addBackArrow(width, height, forBlockDepth);
            }
            calcInnerBlockSize();
            //      :
            if (forBlock.getNext() != null) {
                // ,    
                addArrowToNextBlock(width, height, forBlockDepth, false);
                //  .
                forBlock.getNext().apply(this);
            } else {
                // ,      (   Try),
                //     .
                if (myNestingBlockStack.size() == 2) {
                    if (!myNoNeedArrow) {
                        addArrowToNextBlock(width, height, forBlockDepth, true);
                    }
                    myNoNeedArrow = false;
                } else if (myNestingBlockStack.size() > 2) {
                    addArrowToNextBlock(width, height, forBlockDepth, true);
                }
            }
        }
    }

    private void caseIfBlockIn(IBlock ifBlock) {
        IfBlockDrawer ifbd = new IfBlockDrawer(correctText(ifBlock),
                mySettings);
        myNestingBlockStack.push(new WidthInformation());
        //   IFBlockDrawer'.
        //  -     + 2  (  ).
        //  -     + 2  (  ).
        ifbd.setBounds(myCenter - myCurrentBlockWidth - myHorizontalStandoff,
                myDepth, 2 * (myCurrentBlockWidth + myHorizontalStandoff),
                2 * (myCurrentBlockHeight + myVerticalStandoff));
        //      .
        if (myCurrentBlockWidth + myHorizontalStandoff > myWidth) {
            myWidth = myCurrentBlockWidth + myHorizontalStandoff;
        }
        mySchemePanel.add(ifbd);
    }
    
    //        
    //,  "".
    private void addFirstYesArrow(int width, int height) {
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        int ifArrowCenter = width / 2 + myHorizontalStandoff
                + ViewSettings.IF_ARROW_X + lineWidth;
        List<Point> breaks = new ArrayList<Point>();
        breaks.add(new Point(ViewSettings.IF_ARROW_X,
                height / 2 + myVerticalStandoff));
        breaks.add(new Point(lineWidth, height / 2 + myVerticalStandoff));
        breaks.add(new Point(lineWidth, height + 2 * myVerticalStandoff + myYesArrowStandoff));
        breaks.add(new Point(ifArrowCenter,
                height + 2 * myVerticalStandoff + myYesArrowStandoff));
        breaks.add(new Point(ifArrowCenter,
                height + 2 * myVerticalStandoff + myYesArrowStandoff
                       + ViewSettings.ARROW_LENGTH));
        BrokenArrowDrawer ifad = new BrokenArrowDrawer(
                breaks, new Point(lineWidth, height / 2 + myVerticalStandoff),
                BrokenArrowDrawer.YES,
                BrokenArrowDrawer.VERTICAL, false, mySettings);
        ifad.setBounds(myCenter - ifArrowCenter,
                myDepth,
                ifArrowCenter + ViewSettings.ARROW_WEDTH / 2,
                height + 2 * myVerticalStandoff + myYesArrowStandoff
                    + ViewSettings.ARROW_LENGTH);
        myDepth += height + 2 * myVerticalStandoff + myYesArrowStandoff
                + ViewSettings.ARROW_LENGTH;
        myNestingBlockStack.peek().setLeftWidth(ifArrowCenter);
        mySchemePanel.add(ifad);
    }
    
    //        
    //  "".
    private void addNoArrow(int width, int height, int ifBlockDepth, boolean isSimpleLine) {
        WidthInformation info = myNestingBlockStack.peek();
        int extraWidth = 0;
        if (info.getRightWidth() > width / 2) {
            extraWidth = info.getRightWidth() - width / 2;
        }
        int ifArrowCenter = width / 2 + myHorizontalStandoff
                + ViewSettings.IF_ARROW_X + extraWidth;
        List<Point> breaks = new ArrayList<Point>();
        breaks.add(new Point(ViewSettings.ARROW_WEDTH / 2
                + ifArrowCenter - ViewSettings.IF_ARROW_X - extraWidth,
                height / 2 + myVerticalStandoff));
        breaks.add(new Point(ViewSettings.ARROW_WEDTH / 2
                + ifArrowCenter,
                height / 2 + myVerticalStandoff));
        breaks.add(new Point(ViewSettings.ARROW_WEDTH / 2
                + ifArrowCenter,
                myDepth - ifBlockDepth + myNoArrowStandoff));
        breaks.add(new Point(ViewSettings.ARROW_WEDTH / 2,
                myDepth - ifBlockDepth + myNoArrowStandoff));
        breaks.add(new Point(ViewSettings.ARROW_WEDTH / 2,
                myDepth - ifBlockDepth + myNoArrowStandoff
                        + ViewSettings.ARROW_LENGTH));
        BrokenArrowDrawer ifad = new BrokenArrowDrawer(
                breaks,
                new Point(ViewSettings.ARROW_WEDTH / 2
                        + ifArrowCenter
                        - ViewSettings.IF_ARROW_X - extraWidth,
                        height / 2 + myVerticalStandoff),
                BrokenArrowDrawer.NO,
                BrokenArrowDrawer.VERTICAL, isSimpleLine, mySettings);
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        ifad.setBounds(myCenter - ViewSettings.ARROW_WEDTH / 2,
                ifBlockDepth, ViewSettings.ARROW_WEDTH / 2 + ifArrowCenter + lineWidth,
                myDepth - ifBlockDepth + myNoArrowStandoff + ViewSettings.ARROW_LENGTH);
        if (myCenter +ifArrowCenter + lineWidth > myWidth) {
            myWidth = myCenter +ifArrowCenter + lineWidth;
        }
        myDepth += myNoArrowStandoff + ViewSettings.ARROW_LENGTH;
        if (ifArrowCenter + lineWidth > myNestingBlockStack.peek().getRightWidth()) {
            myNestingBlockStack.peek().setRightWidth(ifArrowCenter + lineWidth);
        }
        mySchemePanel.add(ifad);
    }
    
    //         
    // ,     ""   .
    private void addYesArrow(int width, int height, int ifBlockDepth) {
        WidthInformation info = myNestingBlockStack.peek();
        int extraWidth = 0;
        if (info.getLeftWidth() > width / 2) {
            extraWidth = info.getLeftWidth() - width / 2;
        }
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        int ifArrowCenter = width / 2 + myHorizontalStandoff
                + ViewSettings.IF_ARROW_X + extraWidth + lineWidth;
        List<Point> breaks = new ArrayList<Point>();
        breaks.add(new Point(ViewSettings.IF_ARROW_X + extraWidth,
                height / 2+ myVerticalStandoff));
        breaks.add(new Point(lineWidth,
                height / 2+ myVerticalStandoff));
        breaks.add(new Point(lineWidth,
                myDepth - ifBlockDepth + ViewSettings.POINT_SIZE / 2));
        breaks.add(new Point(ifArrowCenter
                - ViewSettings.POINT_SIZE / 2,
                myDepth - ifBlockDepth
                + ViewSettings.POINT_SIZE / 2));
        BrokenArrowDrawer ifad = new BrokenArrowDrawer(
                breaks, new Point(extraWidth, height / 2+ myVerticalStandoff),
                BrokenArrowDrawer.YES,
                BrokenArrowDrawer.HORIZONTAL, false, mySettings);
        ifad.setBounds(myCenter - ifArrowCenter,
                ifBlockDepth, ifArrowCenter + lineWidth,
                myDepth - ifBlockDepth + ViewSettings.ARROW_WEDTH / 2
                    + ViewSettings.POINT_SIZE / 2);
        myNestingBlockStack.peek().setLeftWidth(ifArrowCenter);
        mySchemePanel.add(ifad);
    }
    
    //        ""
    //   .
    private void addSecondYesArrow(int width, int yesBlockDepth, int yesBlockLeftWidth) {
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        int ifArrowCenter = ViewSettings.IF_ARROW_X 
                + myNestingBlockStack.peek().getLeftWidth() + lineWidth;
        List<Point> breaks = new ArrayList<Point>();
        breaks.add(new Point(ifArrowCenter, 0));
        breaks.add(new Point(ifArrowCenter, myYesArrowStandoff));
        breaks.add(new Point(lineWidth, myYesArrowStandoff));
        breaks.add(new Point(lineWidth,
                myDepth - yesBlockDepth + ViewSettings.POINT_SIZE / 2));
        breaks.add(new Point(ifArrowCenter - ViewSettings.POINT_SIZE / 2,
                myDepth - yesBlockDepth + ViewSettings.POINT_SIZE / 2));
        BrokenArrowDrawer ifad = new BrokenArrowDrawer(
                breaks, null, null,
                BrokenArrowDrawer.HORIZONTAL, false, mySettings);
        ifad.setBounds(myCenter - ifArrowCenter,
                yesBlockDepth, ifArrowCenter + lineWidth,
                myDepth - yesBlockDepth + ViewSettings.ARROW_WEDTH / 2
                        + ViewSettings.POINT_SIZE / 2);
        if (ifArrowCenter > yesBlockLeftWidth) {
            myNestingBlockStack.peek().setLeftWidth(ifArrowCenter);
        };
        mySchemePanel.add(ifad);
    }
    
    public void caseIfBlock(IIfBlock ifBlock) {
        if (ifBlock != null) {
            caseIfBlockIn(ifBlock);
            int width = 2 * myCurrentBlockWidth;
            int height = 2 * myCurrentBlockHeight;
            int ifBlockDepth = myDepth;
            int yesBlockDepth = 0;
            boolean endBlockYes = false;
            //   ,  "",  :
            if (ifBlock.getYesBlock() != null) {
                //   
                addFirstYesArrow(width, height);
                //   
                ifBlock.getYesBlock().apply(this);
                //        IEndBlock
                // ,          .
                endBlockYes = myEndBlockNow;
                yesBlockDepth = myDepth;
            } else {
                //  ,  "",  ,
                //          .
                myDepth += height + 2 * myVerticalStandoff;
            }
            // ,  "".   ,  
            //   ,    ,       
            // .
            addNoArrow(width, height, ifBlockDepth, false);
            boolean endBlockNo = false;
            int yesBlockLeftWidth = myNestingBlockStack.peek().getLeftWidth();
            myNestingBlockStack.peek().setLeftWidth(0);
            //   ,  "",  :
            if (ifBlock.getNoBlock() != null) {
                //  
                ifBlock.getNoBlock().apply(this);
                //        IEndBlock
                // ,       .
                endBlockNo = myEndBlockNow;
                if (!endBlockNo) {
                    addArrowDrawer();
                } else {
                    //       ,
                    //   ,  "", 
                    //     ,  "".
                    myDepth += ViewSettings.ARROW_LENGTH;
                }
            }
            //   ,  "",  :
            if (ifBlock.getYesBlock() != null) {
                //       IEndBlock
                if (!endBlockYes) {
                    //     .
                    addSecondYesArrow(width, yesBlockDepth, yesBlockLeftWidth);
                }
            //  :    
            } else {
                //     ,  "".
                addYesArrow(width, height, ifBlockDepth);
            }
            if (yesBlockLeftWidth > myNestingBlockStack.peek().getLeftWidth()) {
                myNestingBlockStack.peek().setLeftWidth(yesBlockLeftWidth);
            }
            //          IEndBlock,
            //     :
            if (!(endBlockYes && endBlockNo)) {
                //  
                addPointDrawer();
                myEndBlockNow = false;
            }
            calcInnerBlockSize();
            //      :
            if (ifBlock.getNext() != null) {
                // ,   
                addArrowDrawer();
                //   .
                ifBlock.getNext().apply(this);
            }
        }
    }

    public void caseInvocationBlock(IInvocationBlock invocationBlock) {
        if (invocationBlock != null) {
            addBlockDrawer(new InvocationBlockDrawer(correctText(invocationBlock),
                    mySettings));
            if (invocationBlock.getNext() != null) {
                addArrowDrawer();
                invocationBlock.getNext().apply(this);
            }
        }
    }
    
    private void caseSwitchBlockIn(ISwitchBlock switchBlock) {
        IfBlockDrawer ifbd = new IfBlockDrawer(correctText(switchBlock),
                mySettings);
        myNestingBlockStack.push(new WidthInformation());
        //   IFBlockDrawer'.
        //  -     + 2  (  ).
        //  -     + 2  (  ).
        ifbd.setBounds(myCenter - myCurrentBlockWidth - myHorizontalStandoff,
                myDepth, 2 * (myCurrentBlockWidth + myHorizontalStandoff),
                2 * (myCurrentBlockHeight + myVerticalStandoff));
        //      .
        if (myCurrentBlockWidth + myHorizontalStandoff > myWidth) {
            myWidth = myCurrentBlockWidth + myHorizontalStandoff;
        }
        mySchemePanel.add(ifbd);
    }

    private void addCaseArrow(int width, int height, int switchBlockDepth, int caseDepth,
            String caseText) {
        WidthInformation info = myNestingBlockStack.peek();
        int extraWidth = 0;
        if (info.getLeftWidth() > width / 2) {
            extraWidth = info.getLeftWidth() - width / 2;
        }
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        int ifArrowCenter = width / 2 + myHorizontalStandoff
                + ViewSettings.IF_ARROW_X + extraWidth + lineWidth;
        List<Point> breaks = new ArrayList<Point>();
        breaks.add(new Point(ViewSettings.IF_ARROW_X + extraWidth,
                height / 2 + myVerticalStandoff));
        breaks.add(new Point(lineWidth, height / 2 + myVerticalStandoff));
        breaks.add(new Point(lineWidth, myYesArrowStandoff 
                + caseDepth - switchBlockDepth));
        breaks.add(new Point(ifArrowCenter,
                myYesArrowStandoff + caseDepth - switchBlockDepth));
        breaks.add(new Point(ifArrowCenter,
                2 * myYesArrowStandoff + caseDepth - switchBlockDepth));
        BrokenArrowDrawer ifad = new BrokenArrowDrawer(
                breaks, new Point(myHorizontalStandoff,
                        myYesArrowStandoff 
                        + caseDepth - switchBlockDepth),
                caseText,
                BrokenArrowDrawer.VERTICAL, false, mySettings);
        int size = Integer.parseInt(mySettings.getProperty("fontSize"));
        int style = Integer.parseInt(mySettings.getProperty("fontStyle"));
        String name = mySettings.getProperty("fontName");
        Rectangle2D textBounds = new Font(name, style, size).getStringBounds(caseText,
                myFontRenderContext);
        int horizontalBound = (int) Math.round(textBounds.getWidth());
        if (horizontalBound < ifArrowCenter + ViewSettings.ARROW_WEDTH / 2) {
            horizontalBound = ifArrowCenter + ViewSettings.ARROW_WEDTH / 2;
        }
        ifad.setBounds(myCenter - ifArrowCenter, switchBlockDepth,
                horizontalBound, 2 * myYesArrowStandoff + caseDepth 
                - switchBlockDepth);
        mySchemePanel.add(ifad);
    }
   
  
    //     switch ,    
    //case  .
    private void addAlternativeArrow(int width, int height, int switchBlockDepth) {
        WidthInformation info = myNestingBlockStack.peek();
        int extraWidth = 0;
        if (info.getRightWidth() > width / 2) {
            extraWidth = info.getRightWidth() - width / 2;
        }
        int ifArrowCenter = width / 2 + myHorizontalStandoff
                + ViewSettings.IF_ARROW_X + extraWidth;
        List<Point> breaks = new ArrayList<Point>();
        breaks.add(new Point(ifArrowCenter - ViewSettings.IF_ARROW_X - extraWidth,
                height / 2 + myVerticalStandoff));
        breaks.add(new Point(ifArrowCenter, height / 2 + myVerticalStandoff));
        breaks.add(new Point(ifArrowCenter, myDepth - switchBlockDepth 
                - ViewSettings.POINT_SIZE / 2));
        breaks.add(new Point(ViewSettings.POINT_SIZE / 2,
               myDepth - switchBlockDepth - ViewSettings.POINT_SIZE / 2));
        BrokenArrowDrawer ifad = new BrokenArrowDrawer(
                breaks,
                new Point(ifArrowCenter - ViewSettings.IF_ARROW_X - extraWidth,
                        height / 2 + myVerticalStandoff),
                BrokenArrowDrawer.NO,
                BrokenArrowDrawer.HORIZONTAL, false, mySettings);
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        ifad.setBounds(myCenter, switchBlockDepth, ifArrowCenter + lineWidth,
                myDepth - switchBlockDepth - ViewSettings.POINT_SIZE / 2
                + ViewSettings.ARROW_WEDTH / 2);
        if (myCenter +ifArrowCenter + lineWidth > myWidth) {
            myWidth = myCenter +ifArrowCenter + lineWidth;
        }
        if (ifArrowCenter + lineWidth > myNestingBlockStack.peek().getRightWidth()) {
            myNestingBlockStack.peek().setRightWidth(ifArrowCenter + lineWidth);
        }
        mySchemePanel.add(ifad);
    }
    
    public void caseSwitchBlock(ISwitchBlock switchBlock) {
        if (switchBlock != null) {
            caseSwitchBlockIn(switchBlock);
            int width = 2 * myCurrentBlockWidth;
            int height = 2 * myCurrentBlockHeight;
            int switchBlockDepth = myDepth;
            myDepth += height + 2 * myVerticalStandoff;
            boolean isLastCaseEmpty = true;
            myEndBlockNow = false;
            List<Integer> caseDepth = new ArrayList<Integer>(); 
            for (Iterator<ICase> it = switchBlock.cases().iterator();
                    it.hasNext();) {
                ICase c = it.next();
                if (c != null) {
                    caseDepth.add(myDepth);
                    myDepth += myYesArrowStandoff;
                    isLastCaseEmpty = true;
                    if (c.getBlock() != null) {
                        isLastCaseEmpty = false;
                        myDepth += myYesArrowStandoff;
                        c.getBlock().apply(this);
                        if (it.hasNext()) {
                            if (!myEndBlockNow) {
                                addArrowDrawer();
                                myDepth -= myYesArrowStandoff;
                            } else {
                                myEndBlockNow = false;
                            }
                        } else {
                            if (myEndBlockNow) {
                                addArrowDrawer();
                            }
                        }
                    }
                }
            }
            Iterator<Integer> it = caseDepth.iterator();
            for (ICase c: switchBlock.cases()) {
                if (c != null) {
                    addCaseArrow(width, height, switchBlockDepth, it.next(),
                            c.getCase());
                }
            }
            WidthInformation info = myNestingBlockStack.peek();
            int extraWidth = 0;
            if (info.getLeftWidth() > width / 2) {
                extraWidth = info.getLeftWidth() - width / 2;
            }
            info.setLeftWidth(width / 2 + myHorizontalStandoff
                    + ViewSettings.IF_ARROW_X + extraWidth);
            if (width / 2 + myHorizontalStandoff > info.getRightWidth()) {
                info.setRightWidth(width / 2 + myHorizontalStandoff);
            }
            if (!isLastCaseEmpty) {
                addArrowDrawer();
            } else {
                myDepth += myYesArrowStandoff;
            }
            addPointDrawer();
            addAlternativeArrow(width, height, switchBlockDepth);
            calcInnerBlockSize();
            if (switchBlock.getNext() != null) {
                addArrowDrawer();
                switchBlock.getNext().apply(this);
            }
        }
    }
    
    private void addCatchArrow(int width, int height, int tryBlockDepth, int catchDepth) {
        WidthInformation info = myNestingBlockStack.peek();
        int extraWidth = 0;
        if (info.getLeftWidth() > width / 2) {
            extraWidth = info.getLeftWidth() - width / 2;
        }
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        int ifArrowCenter = width / 2 + myHorizontalStandoff
                + ViewSettings.IF_ARROW_X + extraWidth + lineWidth;
        List<Point> breaks = new ArrayList<Point>();
        breaks.add(new Point(ViewSettings.IF_ARROW_X + extraWidth,
                height / 2 + myVerticalStandoff));
        breaks.add(new Point(lineWidth, height / 2 + myVerticalStandoff));
        breaks.add(new Point(lineWidth, catchDepth - tryBlockDepth));
        breaks.add(new Point(ifArrowCenter,
                catchDepth - tryBlockDepth));
        breaks.add(new Point(ifArrowCenter,
                myYesArrowStandoff + catchDepth - tryBlockDepth));
        BrokenArrowDrawer ifad = new BrokenArrowDrawer(
                breaks, null, null,
                BrokenArrowDrawer.VERTICAL, false, mySettings);
        ifad.setBounds(myCenter - ifArrowCenter, tryBlockDepth,
                ifArrowCenter + ViewSettings.ARROW_WEDTH / 2,
                myYesArrowStandoff + catchDepth - tryBlockDepth);
        mySchemePanel.add(ifad);
    }
    
    public void caseTryBlock(ITryBlock tryBlock) {
        if (tryBlock != null) {
            int tryDepth = myDepth;
            addBlockDrawer(new BeginEndBlockDrawer(correctText(tryBlock), mySettings));
            int width = myCurrentBlockWidth;
            int height = myCurrentBlockHeight;
            myNestingBlockStack.push(new WidthInformation());
            if (tryBlock.getFirstBlock() != null) {
                addArrowDrawer();
                tryBlock.getFirstBlock().apply(this);
            }
            List<Integer> catchDepth = new ArrayList<Integer>(); 
            for (Iterator<IBlock> it = tryBlock.getCatch().iterator();
                    it.hasNext();) {
                IBlock cb = it.next();
                addArrowDrawer();
                catchDepth.add(myDepth);
                myDepth += myYesArrowStandoff;
                addBlockDrawer(new BeginEndBlockDrawer(correctText(cb), mySettings));
                if (cb.getNext() != null) {
                    addArrowDrawer();
                    if ((!it.hasNext()) && (tryBlock.getFinally() == null)
                            && (tryBlock.getNext() == null)
                            && (myNestingBlockStack.size() == 2)) {
                        myNoNeedArrow = true;
                        
                    }
                    cb.getNext().apply(this);
                }
            }
            if (tryBlock.getFinally() != null) {
                addArrowDrawer();
                catchDepth.add(myDepth);
                myDepth += myYesArrowStandoff;
                addBlockDrawer(new BeginEndBlockDrawer(correctText(
                        new BlockFactory().createActionBlock("finally")),
                        mySettings));
                addArrowDrawer();
                if ((tryBlock.getNext() == null) && (myNestingBlockStack.size() == 2)) {
                    myNoNeedArrow = true;
                }
                tryBlock.getFinally().apply(this);
            }
            for (Integer cd: catchDepth) {
                addCatchArrow(width, height, tryDepth, cd);
            }
            WidthInformation info = myNestingBlockStack.peek();
            int extraWidth = 0;
            if (info.getLeftWidth() > width / 2) {
                extraWidth = info.getLeftWidth() - width / 2;
            }
            info.setLeftWidth(width / 2 + myHorizontalStandoff
                    + ViewSettings.IF_ARROW_X + extraWidth);
            if (width / 2 + myHorizontalStandoff > info.getRightWidth()) {
                info.setRightWidth(width / 2 + myHorizontalStandoff);
            }
            calcInnerBlockSize();
            if (tryBlock.getNext() != null) {
                addArrowDrawer();
                tryBlock.getNext().apply(this);
            }
        }
    }
    
    private void addWhileBackArrow(int pointDepth) {
        WidthInformation info = myNestingBlockStack.peek();
        int extraWidth = info.getLeftWidth();
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        int ifArrowCenter = myPointSize + ViewSettings.IF_ARROW_X + extraWidth
                + lineWidth;
        List<Point> breaks = new ArrayList<Point>();
        breaks.add(new Point(ifArrowCenter, myDepth - pointDepth));
        breaks.add(new Point(ifArrowCenter, myDepth - pointDepth 
                + myBackArrowStandoff));
        breaks.add(new Point(lineWidth,  myDepth - pointDepth 
                + myBackArrowStandoff));
        breaks.add(new Point(lineWidth,  myPointSize / 2));
        breaks.add(new Point(ifArrowCenter - myPointSize / 2, myPointSize / 2));
        BrokenArrowDrawer ifad = new BrokenArrowDrawer(breaks, null, null,
                BrokenArrowDrawer.HORIZONTAL, false, mySettings);
        ifad.setBounds(myCenter - ifArrowCenter,
                pointDepth, ifArrowCenter + lineWidth,
                myDepth - pointDepth + myBackArrowStandoff + lineWidth);
        myNestingBlockStack.peek().setLeftWidth(ifArrowCenter);
        mySchemePanel.add(ifad);
        info.setLeftWidth(ifArrowCenter);
        
    }
    
    private void addEmptyBackArrow(int pointDepth, int width, int height) {
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        int ifArrowCenter = width / 2 + myHorizontalStandoff
            + ViewSettings.IF_ARROW_X + lineWidth;
        List<Point> breaks = new ArrayList<Point>();
        breaks.add(new Point(ViewSettings.IF_ARROW_X, myDepth - pointDepth
                 + height / 2 + myVerticalStandoff));
        breaks.add(new Point(lineWidth, myDepth - pointDepth
                 + height / 2 + myVerticalStandoff));
        breaks.add(new Point(lineWidth,  myPointSize / 2));
        breaks.add(new Point(ifArrowCenter - myPointSize / 2, myPointSize / 2));
        BrokenArrowDrawer ifad = new BrokenArrowDrawer(
                breaks, new Point(0, myDepth - pointDepth 
                        + height / 2 + myVerticalStandoff),
                BrokenArrowDrawer.YES,
                BrokenArrowDrawer.HORIZONTAL, false, mySettings);
        ifad.setBounds(myCenter - ifArrowCenter, pointDepth,
                ifArrowCenter + lineWidth, myDepth - pointDepth
                + height / 2 + myVerticalStandoff + lineWidth);
        myNestingBlockStack.peek().setLeftWidth(ifArrowCenter);
        mySchemePanel.add(ifad);
    }
    
    public void caseWhileBlock(IWhileBlock whileBlock) {
        if (whileBlock != null) {
            addPointDrawer();
            int pointDepth = myDepth - myPointSize;                       
            addArrowDrawer();
            
            caseIfBlockIn(whileBlock);
            
            int width = 2 * myCurrentBlockWidth;
            int height = 2 * myCurrentBlockHeight;
            int whileBlockDepth = myDepth;
            
            //    
            if (whileBlock.getFirstBlock() != null) {
                // ,     
                addFirstYesArrow(width, height);
                //   .
                whileBlock.getFirstBlock().apply(this);
                //  
                addWhileBackArrow(pointDepth);
            } else {
                addEmptyBackArrow(pointDepth, width, height);
                myDepth += height;
            }
            calcInnerBlockSize();
            //      :
            if (whileBlock.getNext() != null) {
                // ,    
                addNoArrow(width, height, whileBlockDepth, false);
                //  .
                whileBlock.getNext().apply(this);
            } else {
                // ,      (   Try),
                //     .
                if (myNestingBlockStack.size() == 2) {
                    if (!myNoNeedArrow) {
                        addNoArrow(width, height, whileBlockDepth, true);
                    }
                    myNoNeedArrow = false;
                } else if (myNestingBlockStack.size() > 2) {
                    addNoArrow(width, height, whileBlockDepth, true);
                }
            }
        }
    }

}
