package org.amse.ak.schemebuilder.view;

import javax.swing.JPanel;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;

import org.amse.ak.schemebuilder.model.IScheme;
import org.amse.ak.schemebuilder.view.drawer.MethodNameDrawer;
import org.amse.ak.schemebuilder.view.settings.ViewSettings;

public class Layouter {

    private IScheme myScheme;

    private final JPanel mySchemePanel;
    
    private final ViewSettings mySettings;

    private int myDepth = ViewSettings.STANDOFF_BORDER;
    
    private int myWidth = 2 * ViewSettings.CENTER;
    
    private final FontRenderContext myFontRenderContext = ViewSettings.FONT_RENDER_CONTEXT;
    
    private final int myCenter = ViewSettings.CENTER;
    
    private final int myVerticalStandoff = ViewSettings.VERTICAL_STANDOFF_IN_BLOCK
            + ViewSettings.BORDER;

    public Layouter(IScheme scheme, JPanel schemePanel, ViewSettings settings) {
        myScheme = scheme;
        mySchemePanel = schemePanel;
        mySettings = settings;
        mySchemePanel.setLayout(null);
    }

    public IScheme getScheme() {
        return myScheme;
    }
    
    public void setScheme(IScheme scheme) {
        myScheme = scheme;
    }
    
    //   .
    private void addMethodNameDrawer() {
        MethodNameDrawer mnd = new MethodNameDrawer(myScheme.getMethod(),
                mySettings);
        //   ,    .
        int size = Integer.parseInt(mySettings.getProperty("fontSize"));
        int style = Integer.parseInt(mySettings.getProperty("fontStyle"));
        String name = mySettings.getProperty("fontName");
        Font font = new Font(name, style, size);
        Rectangle2D myTextBounds = font.getStringBounds(
                myScheme.getMethod().getClassPath(), myFontRenderContext);
        int width = (int) Math.round(myTextBounds.getWidth());
        int height = (int) Math.round(myTextBounds.getHeight());
        //   .
        myTextBounds = font.getStringBounds(
                myScheme.getMethod().getName(), myFontRenderContext);
        //   -         
        //  .
        if (width < (int) Math.round(myTextBounds.getWidth())) {
            width = (int) Math.round(myTextBounds.getWidth());
        }
        //   -        .
        height += (int) Math.round(myTextBounds.getHeight());
        //    .
        mnd.setBounds(myCenter - width / 2, myDepth, width,
                height + myVerticalStandoff);
        //      .
        if (width / 2 > myWidth) {
            myWidth = width / 2;
        }
        mySchemePanel.add(mnd);
        //   .
        myDepth += height + myVerticalStandoff;
    }
    
    public void pack() {
        if (myScheme != null) {
            if (myScheme.getBeginBlock() != null) {
                mySchemePanel.removeAll();
                myDepth = ViewSettings.STANDOFF_BORDER;
                addMethodNameDrawer();
                LayoutVisitor visitor = new LayoutVisitor(mySchemePanel, mySettings,
                        myDepth, myWidth);
                myScheme.getBeginBlock().apply(visitor);
                mySchemePanel.setPreferredSize(new Dimension(visitor.getWidth() + ViewSettings.STANDOFF_BORDER,
                        visitor.getDepth() + ViewSettings.STANDOFF_BORDER+10));
                mySchemePanel.revalidate();
                mySchemePanel.repaint();
            }
        }
    }

}
