package org.amse.ak.schemebuilder.view;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.tree.DefaultMutableTreeNode;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URL;

import org.amse.ak.schemebuilder.builder.ChooseMethodBuilder;
import org.amse.ak.schemebuilder.builder.SchemeBuilder;
import org.amse.ak.schemebuilder.builder.SchemeBuilderException;
import org.amse.ak.schemebuilder.io.reader.SchemeReader;
import org.amse.ak.schemebuilder.io.reader.SettingsReader;
import org.amse.ak.schemebuilder.io.writer.SchemeWriter;
import org.amse.ak.schemebuilder.io.writer.SettingsWriter;
import org.amse.ak.schemebuilder.model.*;
import org.amse.ak.schemebuilder.model.impl.Method;
import org.amse.ak.schemebuilder.view.settings.ViewSettings;

@SuppressWarnings("serial")
public class MyFrame extends JFrame {

    public JPanel mySchemePanel;

    public IScheme myScheme = null;

    public Layouter myLayouter;
    
    private final ViewSettings mySettings = ViewSettings.getSettings();
    
    private final JFileChooser myFileChooser = new JFileChooser();
    
    private final JavaFileFilter myJavaFileFilter = new JavaFileFilter();
    
    private final JpegFileFilter myJpegFileFilter = new JpegFileFilter();
    
    private final XMLFileFilter myXMLFileFilter = new XMLFileFilter();
    
    private final JScrollPane myScrollPanel;

       
    public MyFrame(IMethod m, int args) {
        super("Scheme Builder");
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        addWindowListener(new WindowListener());
        
        MyMenuBar menuBar = new MyMenuBar();
        MyMenu file = menuBar.createMenu("File");
        file.createMenuItem("Open Block Scheme...", 'O', 'O', KeyEvent.CTRL_MASK,
                new OpenBlockSchemeListener());
        file.createMenuItem("Open Java Sourse...", 'J', 'J', KeyEvent.CTRL_MASK,
                new OpenJavaSourseListener());
        file.createSeparator();
        file.createMenuItem("Save Block Scheme", 'S', 'S', KeyEvent.CTRL_MASK,
                new SaveBlockSchemeListener());
        file.createMenuItem("Save As JPEG...", 'G', 'G', KeyEvent.CTRL_MASK,
                new SaveAsJPEGListener());
        file.createMenuItem("Save As Block Scheme...", new SaveAsBlockSchemeListener());
        file.createSeparator();
        file.createMenuItem("Preferences...", 'P', 'P', KeyEvent.CTRL_MASK,
                new PreferencesListener());
        file.createSeparator();
        file.createMenuItem("Quit", 'Q', 'Q', KeyEvent.CTRL_MASK, new QuitListener());
        MyMenu help = menuBar.createMenu("Help");
        help.createMenuItem("Help", 'H', 'H', KeyEvent.CTRL_MASK, new HelpListener());
        help.createMenuItem("About", 'b', 'B', KeyEvent.CTRL_MASK, new AboutListener());

        setJMenuBar(menuBar);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(createToolBar(), BorderLayout.PAGE_START);
        
        Color background = mySettings.parseColor(mySettings.getProperty("backgroundColor"));
        
        setBackground(background);

        mySchemePanel = new ScrollablePanel();
        mySchemePanel.setBackground(background);
        mySchemePanel.setLayout(null);
        myScrollPanel = new JScrollPane(mySchemePanel);
        getContentPane().add(myScrollPanel,
                BorderLayout.CENTER);

        readSettings();
        myLayouter = new Layouter(myScheme, mySchemePanel, mySettings);
        
        if (m != null) {
             buildScheme(m);
        } else {
            if (args != 0) {
                JOptionPane.showMessageDialog(MyFrame.this, "Invalid number of arguments.",
                        "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    private ImageIcon createIcon(String name) {
        URL iconURL = MyFrame.class.getResource("/org/amse/ak/schemebuilder/view/icon/"
                + name + ".png");
        return new ImageIcon(iconURL);
    }
    
    private JToolBar createToolBar() {
        JToolBar toolBar = new JToolBar(JToolBar.HORIZONTAL);
        toolBar.setFloatable(false);
        toolBar.setBorderPainted(false);
        
        JButton button = new JButton(createIcon("openJavaSourse"));
        button.setToolTipText("Open Java Sourse");
        button.addActionListener(new OpenJavaSourseListener());
        toolBar.add(button);
        
        button = new JButton(createIcon("openBlockScheme"));
        button.setToolTipText("Open Block Scheme");
        button.addActionListener(new OpenBlockSchemeListener());
        toolBar.add(button);
        
        button = new JButton(createIcon("saveAsJPEG"));
        button.setToolTipText("Save As JPEG");
        button.addActionListener(new SaveAsJPEGListener());
        toolBar.add(button);
        
        button = new JButton(createIcon("saveBlockScheme"));
        button.setToolTipText("Save Block Scheme");
        button.addActionListener(new SaveBlockSchemeListener());
        toolBar.add(button);
        
        button = new JButton(createIcon("saveAsBlockScheme"));
        button.setToolTipText("Save As Block Scheme");
        button.addActionListener(new SaveAsBlockSchemeListener());
        toolBar.add(button);
        
        return toolBar;
      }
    
    private void readSettings() {
        SettingsReader sr = new SettingsReader(mySettings);
        sr.read();
    }
    
    private void writeSettings() {
        try {
            SettingsWriter sw = new SettingsWriter(mySettings);
            sw.write();
        } catch (Exception e) {
            
        }
    }
    
    private class WindowListener extends WindowAdapter {
        
        public void windowClosing(WindowEvent e) {
            writeSettings();
            super.windowClosing(e);
        }
        
    }
    
    private void buildScheme(IMethod m) {
        try {
            myScheme = new SchemeBuilder(m).build();
            myLayouter.setScheme(myScheme);
            myLayouter.pack();
        } catch (SchemeBuilderException exception) {
            JOptionPane.showMessageDialog(MyFrame.this, exception.getMessage(),
                    "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    private void setJavaFileFilter(JFileChooser fc){        
        fc.setFileFilter(myJavaFileFilter);
        fc.removeChoosableFileFilter(myJpegFileFilter);
        fc.removeChoosableFileFilter(myXMLFileFilter);
    }
    
    private void setJpegFileFilter(JFileChooser fc){        
        fc.setFileFilter(myJpegFileFilter);
        fc.removeChoosableFileFilter(myJavaFileFilter);
        fc.removeChoosableFileFilter(myXMLFileFilter);
    }
        
    private void setXMLFileFilter(JFileChooser fc){        
        fc.setFileFilter(myXMLFileFilter);
        fc.removeChoosableFileFilter(myJavaFileFilter);
        fc.removeChoosableFileFilter(myJpegFileFilter);
    }
    
    private class OpenJavaSourseListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            myFileChooser.setDialogTitle("Open Java Sourse");
            myFileChooser.setCurrentDirectory(new File(mySettings.getProperty("openJavaSourseDirectory")));
            setJavaFileFilter(myFileChooser);
            int res = myFileChooser.showOpenDialog(MyFrame.this);
            if (res == JFileChooser.APPROVE_OPTION) {
                mySettings.setProperty("openJavaSourseDirectory", myFileChooser.getCurrentDirectory().getPath());  
                File f = myFileChooser.getSelectedFile();
                try {
                    DefaultMutableTreeNode methodTreeModel =
                        new ChooseMethodBuilder(f).build();
                    OpenJavaSourseDialog dialog = 
                        new OpenJavaSourseDialog(MyFrame.this, methodTreeModel);
                    dialog.setVisible(true);
                    dialog.dispose();
                    if (dialog.getMethodName() != null) {
                        String name = dialog.getMethodName();
                        int i = 0;
                        while (name.charAt(i) != ' ') {
                            i++;
                        }
                        buildScheme(new Method(f.getPath(), name));
                    }
                } catch (SchemeBuilderException exception) {
                    JOptionPane.showMessageDialog(MyFrame.this, exception.getMessage(),
                            "Error", JOptionPane.ERROR_MESSAGE);
                }
                            
            }
        }
        
    }
    
    private void openScheme(File f) {
        try {
            myScheme = new SchemeReader(f).build();
            myLayouter.setScheme(myScheme);
            myLayouter.pack();
        } catch (SchemeBuilderException exception) {
            JOptionPane.showMessageDialog(MyFrame.this, exception.getMessage(),
                    "Error", JOptionPane.ERROR_MESSAGE);
        }
    }
    
    private class OpenBlockSchemeListener implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            myFileChooser.setDialogTitle("Open Block Scheme");
            myFileChooser.setCurrentDirectory(new File(mySettings.getProperty("openBlockSchemeDirectory")));
            setXMLFileFilter(myFileChooser);
            int res = myFileChooser.showOpenDialog(MyFrame.this);
            if (res == JFileChooser.APPROVE_OPTION) {
                mySettings.setProperty("openBlockSchemeDirectory", myFileChooser.getCurrentDirectory().getPath());
                File f = myFileChooser.getSelectedFile();
                openScheme(f);
            }
        }
        
    }

    private class SaveBlockSchemeListener implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            if (myScheme.getFile() != null) {
                writeScheme();
            } else {
                (new SaveAsBlockSchemeListener()).actionPerformed(e);
            }
        }
        
    }
      
    private void writeScheme() {
        try {
            SchemeWriter fw = new SchemeWriter(myScheme);
            fw.write();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(MyFrame.this, "Can't write to file.",
                "Save", JOptionPane.ERROR_MESSAGE);
        }
    }
      
    private class SaveAsBlockSchemeListener implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            myFileChooser.setDialogTitle("Save As Block Scheme");
            myFileChooser.setCurrentDirectory(new File(mySettings.getProperty("saveAsBlockSchemeDirectory")));
            setXMLFileFilter(myFileChooser);
            int res = myFileChooser.showSaveDialog(MyFrame.this);
            if (res == JFileChooser.APPROVE_OPTION) {
                mySettings.setProperty("saveAsBlockSchemeDirectory", myFileChooser.getCurrentDirectory().getPath());
                File f = myFileChooser.getSelectedFile();
                if (!f.getName().endsWith(".sch")) {
                    f = new File(f.getPath() + ".sch");
                }
                myScheme.setFile(f);
                writeScheme();
            }
        }
        
    }
    
    private class SaveAsJPEGListener implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            myFileChooser.setDialogTitle("Save As JPEG");
            myFileChooser.setCurrentDirectory(new File(mySettings.getProperty("saveAsJPEGDirectory")));             
            setJpegFileFilter(myFileChooser);
            int res = myFileChooser.showSaveDialog(MyFrame.this);
            if (res == JFileChooser.APPROVE_OPTION) {
                mySettings.setProperty("saveAsJPEGDirectory", myFileChooser.getCurrentDirectory().getPath());
                File f = myFileChooser.getSelectedFile();
                if (!f.getName().endsWith(".jpg")) {
                    f = new File(f.getPath() + ".jpg");
                }
                BufferedImage image = new BufferedImage(mySchemePanel.getWidth(),
                        mySchemePanel.getHeight(), BufferedImage.TYPE_INT_RGB);
                Graphics g = image.createGraphics();
                mySchemePanel.print(g);
                try {
                    ImageIO.write(image, "jpg", f);
                } catch (IOException exception) {
                    JOptionPane.showMessageDialog(MyFrame.this, "Couldn't export scheme.",
                            "Error", JOptionPane.ERROR_MESSAGE);
                }
            }
        }
        
    }

    private class QuitListener implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            writeSettings();
            dispose();
        }
        
    }

    private class PreferencesListener implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            PreferencesDialog dialog = 
                new PreferencesDialog(MyFrame.this, mySettings);
            dialog.setVisible(true);
            dialog.dispose();
        }
        
    }
    
    private class HelpListener implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            JOptionPane.showMessageDialog(MyFrame.this,
                    "If you want to merge some part of your method in one block, you can use tags:\n"
                    + "insert '//@begin@TagName' befor merging part and '//@end' after in your Java file.\n"
                    + "Example:\n//@begin@MyCycle\nfor(int i = 0; i , n; i++) {\n"
                    + "...doing something\n}\n//@end\n\n"
                    + "Warning: if there is a line comment, which begins with '@', in your class,\n"
                    + "your class wouldn't be parsed.\n\n" 
                    + "Warning: do not use nesting tags!\n\n"
                    + "Also you can use command line's arguments for building schemes:\n"
                    + "first argument is a class path, then write method name in such way:\n"
                    + "methodName ( )\n"
                    + "or\n"
                    + "methodName (FirstType firstArgument , SecondType secondArgument )\n\n"
                    + "Warning: don't forget white spaces before commas and brackets in the method name!\n"
                    + "Also insert white spaces if there is a generic type int your method:\n"
                    + "List < String >\n\n"
                    + "Warning: unfortunally, current version of this application build wrong\n"
                    + "schemes if your class has nesting class and they have the same methods,\n"
                    + "schemes for this methods will be built uncorrect.",
                    "Help", JOptionPane.PLAIN_MESSAGE);
        }
        
    }
    
    private class AboutListener implements ActionListener {
     
        public void actionPerformed(ActionEvent e) {
            JOptionPane.showMessageDialog(MyFrame.this,
                    "Scheme Builder\nVersion 1.0\n\nCopyright (c) 2007-2008 "
                    + "by Konstantin Aleev\n\nMy e-mail: konstantin.aleev@gmail.com",
                    "About", JOptionPane.PLAIN_MESSAGE);
        }
        
    }
    
    public JPanel getSchemePanel() {
        return mySchemePanel;
    }
    
    public void setScheme(IScheme scheme) {
        myScheme = scheme;
        myLayouter.setScheme(scheme);
        myLayouter.pack();
    }
    
    public void changeBackGroundColor(Color color) {
        getContentPane().setBackground(color);
        mySchemePanel.setBackground(color);
    }
    
    public void repaintScheme() {
        myLayouter.pack();
        mySchemePanel.repaint();
    }

}
