package org.amse.ak.schemebuilder.view;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

@SuppressWarnings("serial")
class OpenJavaSourseDialog extends JDialog {
    
    private final static int X_SIZE = 300;
    
    private final static int Y_SIZE = 200;
    
    private final JTree myTree;
    
    private String myMethodName;
    
    public OpenJavaSourseDialog(Frame owner, DefaultMutableTreeNode methodTreeModel) {
        super(owner, "Choose method", true);
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        
        myTree = new JTree(methodTreeModel);
        myTree.getSelectionModel().setSelectionMode(
                TreeSelectionModel.SINGLE_TREE_SELECTION);
        Object[] path = {methodTreeModel, methodTreeModel.getFirstChild()};
        myTree.getSelectionModel().addSelectionPath(new TreePath(path));
        myTree.setVisibleRowCount(10);
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(new JScrollPane(myTree), BorderLayout.CENTER);
        
        myTree.addMouseListener(new DoubleClickListener());
        
        JPanel gridButtonPanel = new JPanel();
        gridButtonPanel.setLayout(new GridLayout(1, 2, 5, 0));
        JButton okButton = new JButton("Ok");
        okButton.addActionListener(new OkListener());
        gridButtonPanel.add(okButton);
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new CancelListener());
        gridButtonPanel.add(cancelButton);
        JPanel flowButtonPanel = new JPanel();
        flowButtonPanel.setLayout(new FlowLayout());
        flowButtonPanel.add(gridButtonPanel);
        getContentPane().add(flowButtonPanel, BorderLayout.PAGE_END);
        
        setSize(X_SIZE, Y_SIZE);        
        Point p = owner.getLocation();
        p.translate((owner.getWidth() - X_SIZE) / 2,
                (owner.getHeight() - Y_SIZE) / 2);
        this.setLocation(p);
    }
    
    public String getMethodName() {
        return myMethodName;
    }
    
    
    private class DoubleClickListener extends MouseAdapter {
        
        public void mouseClicked(MouseEvent e) {
            TreePath p = myTree.getPathForLocation(e.getX(), e.getY());
            if ((e.getClickCount() == 2) && (e.getButton() == MouseEvent.BUTTON1)) {
                if ((p != null) && (p.getPathCount() == 2)){
                    myMethodName = p.getLastPathComponent().toString();
                    dispose(); 
                }
            }
        }
        
    }
    
    private class OkListener implements ActionListener {
     
        public void actionPerformed(ActionEvent e) {
            TreePath p = myTree.getLeadSelectionPath();
            if ((p != null) && (p.getPathCount() == 2)){
                myMethodName = p.getLastPathComponent().toString();
                dispose(); 
            } else {
                JOptionPane.showMessageDialog(OpenJavaSourseDialog.this, "You should select method!",
                        "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
        
    }
    
    private class CancelListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            dispose();        
        }
    }
    
}
