package org.amse.ak.schemebuilder.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.amse.ak.schemebuilder.view.settings.ViewSettings;

@SuppressWarnings("serial")
public class PreferencesDialog extends JDialog {
    
    private final static int X_SIZE = 450;
    
    private final static int Y_SIZE = 250;
    
    private final MyFrame myOwner;
    
    private final ViewSettings mySettings;
    
    private Color myBackgroundColor;
    
    private Color myBlockColor;
    
    private Color myLineColor;
    
    private Color myTextColor;
    
    private JTextField myLineField;
    
    private JTextField myFontSizeField;
    
    private JTextField myFontStyleField;
    
    private JTextField myBlockField;
    
    public PreferencesDialog(MyFrame owner, ViewSettings settings) {
        super(owner, "Preferences", true);
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        
        myOwner = owner;
        mySettings = settings;
        myBackgroundColor = mySettings.parseColor(mySettings.getProperty("backgroundColor"));
        myBlockColor = mySettings.parseColor(mySettings.getProperty("blockColor"));
        myLineColor = mySettings.parseColor(mySettings.getProperty("lineColor"));
        myTextColor = mySettings.parseColor(mySettings.getProperty("textColor"));
        
        getContentPane().setLayout(new BorderLayout());
        
        JPanel preferencesPanel = new JPanel();
        preferencesPanel.setLayout(new GridLayout(4, 3, 5, 12));
        
        JButton backgroundButton = new JButton("Background color");
        backgroundButton.setForeground(myBackgroundColor);
        backgroundButton.addActionListener(new BackgroundColorListener());
        preferencesPanel.add(backgroundButton);
        
        JPanel flow = new JPanel();
        flow.setLayout(new BorderLayout());
        flow.add(new JLabel("Line width:"), BorderLayout.EAST);
        myLineField = new JTextField(mySettings.getProperty("lineWidth"), 4);
        preferencesPanel.add(flow);
        preferencesPanel.add(myLineField);
        
        JButton blockButton = new JButton("Block color");
        blockButton.setForeground(myBlockColor);
        blockButton.addActionListener(new BlockColorListener());
        preferencesPanel.add(blockButton);
        
        flow = new JPanel();
        flow.setLayout(new BorderLayout());
        flow.add(new JLabel("Font size:"), BorderLayout.EAST);
        myFontSizeField = new JTextField(mySettings.getProperty("fontSize"), 4);
        preferencesPanel.add(flow);
        preferencesPanel.add(myFontSizeField);
        
        JButton lineButton = new JButton("Line color");
        lineButton.setForeground(myLineColor);
        lineButton.addActionListener(new LineColorListener());
        preferencesPanel.add(lineButton);
        
        flow = new JPanel();
        flow.setLayout(new BorderLayout());
        flow.add(new JLabel("Font style:"), BorderLayout.EAST);
        myFontStyleField = new JTextField(mySettings.getProperty("fontStyle"), 4);
        preferencesPanel.add(flow);
        preferencesPanel.add(myFontStyleField);
        
        JButton textButton = new JButton("Text color");
        textButton.setForeground(myTextColor);
        textButton.addActionListener(new TextColorListener());
        preferencesPanel.add(textButton);
        
        flow = new JPanel();
        flow.setLayout(new BorderLayout());
        flow.add(new JLabel("Max block width:"), BorderLayout.EAST);
        myBlockField = new JTextField(mySettings.getProperty("maxBlockWidth"), 4);
        preferencesPanel.add(flow);
        preferencesPanel.add(myBlockField);
        
        JPanel flowPreferencesPanel = new JPanel();
        flowPreferencesPanel.setLayout(new FlowLayout());
        flowPreferencesPanel.add(preferencesPanel);
        getContentPane().add(flowPreferencesPanel, BorderLayout.CENTER);
        
        JPanel gridButtonPanel = new JPanel();
        gridButtonPanel.setLayout(new GridLayout(1, 2, 5, 0));
        JButton okButton = new JButton("Ok");
        okButton.addActionListener(new OkListener());
        gridButtonPanel.add(okButton);
        JButton applyButton = new JButton("Apply");
        applyButton.addActionListener(new ApplyListener());
        gridButtonPanel.add(applyButton);
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new CancelListener());
        gridButtonPanel.add(cancelButton);
        JPanel flowButtonPanel = new JPanel();
        flowButtonPanel.setLayout(new FlowLayout());
        flowButtonPanel.add(gridButtonPanel);
        getContentPane().add(flowButtonPanel, BorderLayout.PAGE_END);
        
        setSize(X_SIZE, Y_SIZE);        
        Point p = owner.getLocation();
        p.translate((owner.getWidth() - X_SIZE) / 2,
                (owner.getHeight() - Y_SIZE) / 2);
        this.setLocation(p);
    }
   
    private class BackgroundColorListener implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            Color color = JColorChooser.showDialog(PreferencesDialog.this,
                    "Choose Background Color", myBackgroundColor);
            if (color != null) {
                myBackgroundColor = color;
                ((JButton) e.getSource()).setForeground(color);
            }
        }
        
    }
    
    private class BlockColorListener implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            Color color = JColorChooser.showDialog(PreferencesDialog.this,
                    "Choose Block Color", myBlockColor);
            if (color != null) {
                myBlockColor = color;
                ((JButton) e.getSource()).setForeground(color);
            }
        }
        
    }
    
    private class LineColorListener implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            Color color = JColorChooser.showDialog(PreferencesDialog.this,
                    "Choose Line Color", myLineColor);
            if (color != null) {
                myLineColor = color;
                ((JButton) e.getSource()).setForeground(color);
            }
        }
        
    }
    
    private class TextColorListener implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            Color color = JColorChooser.showDialog(PreferencesDialog.this,
                    "Choose Text Color", myTextColor);
            if (color != null) {
                myTextColor = color;
                ((JButton) e.getSource()).setForeground(color);
            }
        }
        
    }
    
    private void applyChanges() {
        mySettings.setProperty("lineWidth", myLineField.getText());
        mySettings.setProperty("fontSize", myFontSizeField.getText());
        mySettings.setProperty("fontStyle", myFontStyleField.getText());
        mySettings.setProperty("maxBlockWidth", myBlockField.getText());
        mySettings.setProperty("backgroundColor", myBackgroundColor.getRed() + " "
                + myBackgroundColor.getGreen() + " " + myBackgroundColor.getBlue());
        myOwner.changeBackGroundColor(myBackgroundColor);
        mySettings.setProperty("blockColor", myBlockColor.getRed() + " "
                + myBlockColor.getGreen() + " " + myBlockColor.getBlue());
        mySettings.setProperty("lineColor", myLineColor.getRed() + " "
                + myLineColor.getGreen() + " " + myLineColor.getBlue());
        mySettings.setProperty("textColor", myTextColor.getRed() + " "
                + myTextColor.getGreen() + " " + myTextColor.getBlue());
        myOwner.repaintScheme();
    }
    
    private class OkListener implements ActionListener {
     
        public void actionPerformed(ActionEvent e) {
            applyChanges();
            dispose();
        }
        
    }
    
    private class ApplyListener implements ActionListener {
        
        public void actionPerformed(ActionEvent e) {
            applyChanges();
        }
        
    }
    
    private class CancelListener implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            dispose();        
        }
    }

}
