package org.amse.ak.schemebuilder.view.drawer;

import javax.swing.JComponent;

import java.awt.BasicStroke;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.util.Collections;
import java.util.List;

import org.amse.ak.schemebuilder.view.settings.ViewSettings;

@SuppressWarnings("serial")
public abstract class BlockDrawer extends JComponent {

    private final List<String> myLines;
    
    private final ViewSettings mySettings;
    
    private int myWidth;
    
    private int myHeight;

    private final int myHorizontalStandoff = ViewSettings.HORIZONTAL_STANDOFF_IN_BLOCK;
    
    private final int myVerticalStandoff = ViewSettings.VERTICAL_STANDOFF_IN_BLOCK;
    
    private final int myBorder = ViewSettings.BORDER;

    public BlockDrawer(List<String> lines, ViewSettings settings) {
        super();
        myLines = lines;
        mySettings = settings;
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        FontRenderContext frc = ViewSettings.FONT_RENDER_CONTEXT;
        myWidth = 0;
        myHeight = 0;
        for (String line: myLines) {
            int size = Integer.parseInt(mySettings.getProperty("fontSize"));
            int style = Integer.parseInt(mySettings.getProperty("fontStyle"));
            String name = mySettings.getProperty("fontName");
            Rectangle2D myLineBounds = new Font(name, style, size).getStringBounds(line, frc);
            int lineWidth = (int) Math.round(myLineBounds.getWidth());
            int lineHeight = (int) Math.round(myLineBounds.getHeight());
            if (lineWidth > myWidth) {
                myWidth = lineWidth;
            }
            myHeight += lineHeight;
        }
        paintBlock(g);
        paintText(g);
    }

    protected void paintText(Graphics g) {
        FontRenderContext frc = ViewSettings.FONT_RENDER_CONTEXT;
        myWidth = 0;
        myHeight = 0;
        g.setColor(mySettings.parseColor(mySettings.getProperty("textColor")));
        int size = Integer.parseInt(mySettings.getProperty("fontSize"));
        int style = Integer.parseInt(mySettings.getProperty("fontStyle"));
        String name = mySettings.getProperty("fontName");
        g.setFont(new Font(name, style, size));
        for (String line: myLines) {
            Rectangle2D myLineBounds = g.getFont().getStringBounds(line, frc);
            int lineWidth = (int) Math.round(myLineBounds.getWidth());
            int lineHeight = (int) Math.round(myLineBounds.getHeight());
            if (lineWidth > myWidth) {
                myWidth = lineWidth;
            }
            myHeight += lineHeight;
            g.drawString(line, myHorizontalStandoff + myBorder,
                    myHeight + myBorder);
        }
    }
    
    protected void paintBlock(Graphics g) {
        g.setColor(mySettings.parseColor(mySettings.getProperty("blockColor")));
        int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
        int extra = 0;
        if (lineWidth / 2 == 0) {
            extra = 1;
        }
        g.fillRect(lineWidth / 2, lineWidth / 2, myWidth + 2 * myHorizontalStandoff + lineWidth / 2 + extra,
                myHeight + 2 * myVerticalStandoff + lineWidth / 2 + extra);
        g.setColor(mySettings.parseColor(mySettings.getProperty("lineColor")));
        ((Graphics2D) g).setStroke(new BasicStroke(lineWidth));
        g.drawRect(lineWidth / 2, lineWidth / 2, myWidth + 2 * myHorizontalStandoff + lineWidth / 2 + extra,
                myHeight + 2 * myVerticalStandoff + lineWidth / 2 + extra);
    }
    
    public int getTextWidth() {
        return myWidth;
    }
    
    public void setTextWidth(int width) {
        myWidth = width;
    }
    
    public int getTextHeight() {
        return myHeight;
    }
    
    public void setTextHeight(int height) {
        myHeight = height;
    }
    
    public int getHorizontalStandoff() {
        return myHorizontalStandoff;
    }
    
    public int getVerticalStandoff() {
        return myVerticalStandoff;
    }
    
    public int getMyBorder() {
        return myBorder;
    }
    
    public List<String> lines() {
        return Collections.unmodifiableList(myLines);
    }
    
    public ViewSettings getSettings() {
        return mySettings;
    }

}
