package org.amse.ak.schemebuilder.view.drawer;

import javax.swing.JComponent;

import java.awt.BasicStroke;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.util.Iterator;
import java.util.List;

import org.amse.ak.schemebuilder.view.settings.ViewSettings;

@SuppressWarnings("serial")
public class BrokenArrowDrawer extends JComponent {

    private final List<Point> myBreaks;

    private final Point myTextPoint;

    private final String myText;

    private final int myOrientation;
    
    private final ViewSettings mySettings;

    private final int myWedth = ViewSettings.ARROW_WEDTH;

    private final int myStandoff = ViewSettings.ARROW_STANDOFF;

    private final boolean myIsSimpleLine;
    
    public static final String YES = "Yes";

    public static final String NO = "No";

    public static final int VERTICAL = 0;

    public static final int HORIZONTAL = 1;

    public BrokenArrowDrawer(List<Point> breaks, Point textPoint, String text,
            int orientation, boolean isSimpleLine, ViewSettings settings) {
        myBreaks = breaks;
        myTextPoint = textPoint;
        myText = text;
        myOrientation = orientation;
        mySettings = settings;
        myIsSimpleLine = isSimpleLine;
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        if (myText != null) {
            g.setColor(mySettings.parseColor(mySettings.getProperty("textColor")));
            int size = Integer.parseInt(mySettings.getProperty("fontSize"));
            int style = Integer.parseInt(mySettings.getProperty("fontStyle"));
            String name = mySettings.getProperty("fontName");
            g.setFont(new Font(name, style, size));
            g.drawString(myText, myTextPoint.x, myTextPoint.y);
        }
        if (myBreaks != null) {
            if (myBreaks.size() > 1) {
                g.setColor(mySettings.parseColor(mySettings.getProperty("lineColor")));
                int lineWidth = Integer.parseInt(mySettings.getProperty("lineWidth"));
                ((Graphics2D) g).setStroke(new BasicStroke(lineWidth));
                Iterator<Point> it = myBreaks.iterator();
                Point first = it.next();
                while (true) {
                    Point second = it.next();
                    g.drawLine(first.x, first.y, second.x, second.y);
                    if (!it.hasNext()) {
                        if (!myIsSimpleLine) {
                            if (myOrientation == VERTICAL) {
                                g.drawLine(second.x, second.y, second.x - myWedth / 2,
                                        second.y - myStandoff);
                                g.drawLine(second.x, second.y, second.x + myWedth / 2,
                                        second.y - myStandoff);
                            } else {
                                if (second.x >= first.x) {
                                    g.drawLine(second.x, second.y, second.x - myStandoff,
                                            second.y - myWedth / 2);
                                    g.drawLine(second.x, second.y, second.x - myStandoff,
                                            second.y + myWedth / 2);
                                } else {
                                    g.drawLine(second.x, second.y, second.x + myStandoff,
                                            second.y - myWedth / 2);
                                    g.drawLine(second.x, second.y, second.x + myStandoff,
                                            second.y + myWedth / 2);
                                }
                            }
                        }
                        break;
                    }
                    first = second;
                }

            }
        }
    }

}
