package org.amse.ak.schemebuilder.view.drawer;

import java.awt.BasicStroke;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.util.List;

import org.amse.ak.schemebuilder.view.settings.ViewSettings;

@SuppressWarnings("serial")
public class IfBlockDrawer extends BlockDrawer {

    public IfBlockDrawer(List<String> lines, ViewSettings settings) {
        super(lines, settings);
    }
    
    protected void paintText(Graphics g) {
        FontRenderContext frc = ViewSettings.FONT_RENDER_CONTEXT;
        int width = 0;
        int height = 0;
        List<String> lines = lines();
        for (String line: lines) {
            Rectangle2D myLineBounds = g.getFont().getStringBounds(line, frc);
            int lineWidth = (int) Math.round(myLineBounds.getWidth());
            int lineHeight = (int) Math.round(myLineBounds.getHeight());
            if (lineWidth > width) {
                width = lineWidth;
            }
            height += lineHeight;
        }
        int currentHeight = 0;
        g.setColor(getSettings().parseColor(getSettings().getProperty("textColor")));
        int size = Integer.parseInt(getSettings().getProperty("fontSize"));
        int style = Integer.parseInt(getSettings().getProperty("fontStyle"));
        String name = getSettings().getProperty("fontName");
        g.setFont(new Font(name, style, size));
        for (String line: lines) {
            Rectangle2D myLineBounds = g.getFont().getStringBounds(line, frc);
            int lineHeight = (int) Math.round(myLineBounds.getHeight());
            currentHeight += lineHeight;
            g.drawString(line, getHorizontalStandoff() + getMyBorder() + width / 2,
                    currentHeight  + getMyBorder() + height / 2);
        }
        setTextWidth(width);
        setTextHeight(height);
    }
    
    protected void paintBlock(Graphics g) {
        int width = 2 * getTextWidth() + 2 * getHorizontalStandoff() + getMyBorder();
        int height = 2 * getTextHeight() + 2 * getVerticalStandoff() + getMyBorder();
        int[] xPoints = {0, width / 2, width, width / 2};
        int[] yPoints = {height / 2, 0, height / 2, height};
        Polygon diamond = new Polygon(xPoints, yPoints, 4);
        g.setColor(getSettings().parseColor(getSettings().getProperty("blockColor")));
        g.fillPolygon(diamond);
        g.setColor(getSettings().parseColor(getSettings().getProperty("lineColor")));
        int lineWidth = Integer.parseInt(getSettings().getProperty("lineWidth"));
        ((Graphics2D) g).setStroke(new BasicStroke(lineWidth));
        g.drawPolygon(diamond);
    }

}
