package org.amse.ak.schemebuilder.view.settings;

import java.awt.Color;
import java.awt.font.FontRenderContext;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

public class ViewSettings {

    //Drawing block scheme settings:
    public static final int POINT_SIZE = 7;

    public static final int CENTER = 400;

    public static final int STANDOFF_BORDER = 15;

    public static final int STANDOFF_IN_BLOCK = 5;
    
    public static final int HORIZONTAL_STANDOFF_IN_BLOCK = 5;
    
    public static final int VERTICAL_STANDOFF_IN_BLOCK = 2;
    
    public static final FontRenderContext FONT_RENDER_CONTEXT = new FontRenderContext(
            null, false, false);

    public static final int ARROW_LENGTH = 20;

    public static final int ARROW_WEDTH = 10;

    public static final int ARROW_STANDOFF = 5;

    public static final int IF_ARROW_X = 20;
    
    public static final int FOR_BREAST = 5;
    
    public static final int NO_ARROW_STANDOFF = 30;
    
    public static final int YES_ARROW_STANDOFF = 15;
    
    public static final int BACK_ARROW_STANDOFF = 15;
    
    public static final int NEXT_ARROW_STANDOFF = 30;
    
    public static final int BORDER = 1;
    
    
    private static final ViewSettings SETTINGS = new ViewSettings();
    
     
    private final Map<String, String> myDefaultProperties = new HashMap<String, String>();
    {
        myDefaultProperties.put("lineWidth", "1");
        myDefaultProperties.put("maxBlockWidth", "150");
        myDefaultProperties.put("lineColor", "0 0 0");
        myDefaultProperties.put("textColor", "0 0 0");
        myDefaultProperties.put("backgroundColor", "238 238 238");
        myDefaultProperties.put("blockColor", "255 255 255");
        myDefaultProperties.put("fontSize", "14");
        myDefaultProperties.put("fontStyle", "0");
        myDefaultProperties.put("fontName", "Dialog");
        myDefaultProperties.put("openJavaSourseDirectory", System.getProperty("user.dir"));
        myDefaultProperties.put("openBlockSchemeDirectory", System.getProperty("user.dir"));
        myDefaultProperties.put("saveAsBlockSchemeDirectory", System.getProperty("user.dir"));
        myDefaultProperties.put("saveAsJPEGDirectory", System.getProperty("user.dir"));
    }
    
    private final Map<String, String> myProperties;
    
    private ViewSettings() {
        myProperties = new HashMap<String, String>();  
        myProperties.putAll(myDefaultProperties);
    }
    
    public static ViewSettings getSettings() {
        return SETTINGS;
    }
    
    public String getProperty(String property) {
        return myProperties.get(property);
    }
    
    public void setProperty(String property, String value) {
        myProperties.put(property, value);
    }
    
    public void setDefaultSettings() {
        myProperties.putAll(myDefaultProperties);
    }
    
    public Set<String> getAllProperties() {
        return Collections.unmodifiableSet(myProperties.keySet());
    }
    
    public Color parseColor(String scolor) {
        int redIndex = 1;
        if (scolor.charAt(redIndex) != ' ') {
            if (scolor.charAt(redIndex + 1) != ' ') {
                redIndex = redIndex + 2;
            } else {
                redIndex = redIndex + 1;
            }
        }
        int red = Integer.parseInt(scolor.substring(0, redIndex));
        int greenIndex = redIndex + 2;
        if (scolor.charAt(greenIndex) != ' ') {
            if (scolor.charAt(greenIndex + 1) != ' ') {
                greenIndex = greenIndex + 2;
            } else {
                greenIndex = greenIndex + 1;
            }
        }
        int green = Integer.parseInt(scolor.substring(redIndex + 1, greenIndex));
        int blueIndex = greenIndex + 2;
        if (blueIndex != scolor.length()) {
            if (blueIndex + 1 != scolor.length()) {
                blueIndex = blueIndex + 2;
            } else {
                blueIndex = blueIndex + 1;
            }
        }
        int blue = Integer.parseInt(scolor.substring(greenIndex + 1, blueIndex));
        return new Color(red, green, blue);
    }
}
