package test;

import java.io.File;

import org.amse.ak.schemebuilder.builder.SchemeBuilder;
import org.amse.ak.schemebuilder.builder.SchemeBuilderException;
import org.amse.ak.schemebuilder.model.IBlock;
import org.amse.ak.schemebuilder.model.IBlockFactory;
import org.amse.ak.schemebuilder.model.IIfBlock;
import org.amse.ak.schemebuilder.model.IScheme;
import org.amse.ak.schemebuilder.model.impl.BlockFactory;
import org.amse.ak.schemebuilder.model.impl.Method;
import org.amse.ak.schemebuilder.model.impl.Scheme;

import junit.framework.TestCase;

public class BuilderTests extends TestCase {
    
    private IScheme scheme;

    private final IBlockFactory factory = new BlockFactory();
    
    private final String testFolderPath = "test" + File.separator;
    
    public void setUp() {
        scheme = new Scheme(new Method(null, "test ( ) "));
    }
    
    public void testSchemeWithSimpleStatement() {
        IBlock b = factory.createActionBlock("a = a + 1 ");
        scheme.getBeginBlock().setNext(b);
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                    testFolderPath + "TestSchemeWithSimpleStatement.java",
                    "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            e.printStackTrace();
            fail();
        }
    }
    
    public void testSchemeWithDeclaration() {
        IBlock b = factory.createActionBlock("int a = 1 ");
        scheme.getBeginBlock().setNext(b);
        b.setNext(factory.createActionBlock("a ++ "));
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                    testFolderPath + "TestSchemeWithDeclaration.java",
                    "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            fail();
        }
    }
    
    public void testSchemeWithTwoStatements() {
        IBlock b = factory.createActionBlock("a = a + 1 ");
        scheme.getBeginBlock().setNext(b);
        b.setNext(factory.createActionBlock("a = a + 2 "));
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                    testFolderPath + "TestSchemeWithTwoStatements.java",
                    "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            fail();
        }
    }
    
    public void testSchemeWithReturn() {
        scheme.getBeginBlock().setNext(factory.createEndBlock("return"));
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                    testFolderPath + "TestSchemeWithReturn.java",
                    "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            fail();
        }
    }
    
    public void testSchemeWithShortIf() {
        IIfBlock ifb = factory.createIfBlock("a > 1 ");
        scheme.getBeginBlock().setNext(ifb);
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                    testFolderPath + "TestSchemeWithShortIf.java",
                    "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            fail();
        }
    }

    public void testSchemeShortWithIfAndStatement() {
        IIfBlock ifb = factory.createIfBlock("a > 1 ");
        scheme.getBeginBlock().setNext(ifb);
        ifb.setNext(factory.createActionBlock("a = a + 1 "));
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                    testFolderPath + "TestSchemeWithShortIfAndStatement.java",
                    "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            fail();
        }
    }

    public void testSchemeWithIf() {
        IBlock b = factory.createActionBlock("a = a + 1 ");
        scheme.getBeginBlock().setNext(b);
        IIfBlock ifb = factory.createIfBlock("a > 1 ");
        b.setNext(ifb);
        ifb.setYesBlock(factory.createActionBlock("a = a - 1 "));
        ifb.setNoBlock(factory.createActionBlock("a = 1 - a "));
        b = factory.createActionBlock("a = a + 2 ");
        ifb.setNext(b);
        b.setNext(factory.createEndBlock("return"));
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                    testFolderPath + "TestSchemeWithIf.java",
                    "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            fail();
        }
    }
    
    public void testSchemeWithTwoIf() {
        IBlock b = factory.createActionBlock("a = a + 1 ");
        scheme.getBeginBlock().setNext(b);
        IIfBlock ifb = factory.createIfBlock("a > 1 ");
        b.setNext(ifb);
        ifb.setYesBlock(factory.createActionBlock("a = a - 1 "));
        ifb.setNoBlock(factory.createActionBlock("a = 1 - a "));
        IIfBlock ifb1 = factory.createIfBlock("a > 1 ");
        ifb.setNext(ifb1);
        ifb1.setYesBlock(factory.createActionBlock("a = a - 1 "));
        ifb1.setNoBlock(factory.createActionBlock("a = 1 - a "));
        b = factory.createActionBlock("a = a + 2 ");
        ifb1.setNext(b);
        b.setNext(factory.createEndBlock("return"));
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                testFolderPath + "TestSchemeWithTwoIf.java",
                "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            fail();
        }
    }
    
    public void testSchemeWithNestedIf1() {
        IBlock b = factory.createActionBlock("a = a + 1 ");
        scheme.getBeginBlock().setNext(b);
        IIfBlock ifb = factory.createIfBlock("a > 1 ");
        b.setNext(ifb);
        IIfBlock ifb1 = factory.createIfBlock("a > 1 ");
        ifb1.setYesBlock(factory.createActionBlock("a = a - 1 "));
        ifb1.setNoBlock(factory.createActionBlock("a = 1 - a "));
        ifb.setYesBlock(ifb1);
        ifb.setNoBlock(factory.createActionBlock("a = 1 - a "));
        b = factory.createActionBlock("a = a + 2 ");
        ifb.setNext(b);
        b.setNext(factory.createEndBlock("return"));
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                    testFolderPath + "TestSchemeWithNestedIf1.java",
                    "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            fail();
        }
    }
    
    public void testSchemeWithNestedIf2() {
        IBlock b = factory.createActionBlock("a = a + 1 ");
        scheme.getBeginBlock().setNext(b);
        IIfBlock ifb = factory.createIfBlock("a > 1 ");
        b.setNext(ifb);
        b = factory.createActionBlock("a = a - 1 ");
        ifb.setYesBlock(b);
        IIfBlock ifb1 = factory.createIfBlock("a > 1 ");
        ifb1.setYesBlock(factory.createActionBlock("a = a - 1 "));
        ifb1.setNoBlock(factory.createActionBlock("a = 1 - a "));
        b.setNext(ifb1);
        ifb.setNoBlock(factory.createActionBlock("a = 1 - a "));
        b = factory.createActionBlock("a = a + 2 ");
        ifb.setNext(b);
        b.setNext(factory.createEndBlock("return"));
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                    testFolderPath + "TestSchemeWithNestedIf2.java",
                    "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            fail();
        }
    }
    
    public void testSchemeWithNestedIf3() {
        IBlock b = factory.createActionBlock("a = a + 1 ");
        scheme.getBeginBlock().setNext(b);
        IIfBlock ifb = factory.createIfBlock("a > 1 ");
        b.setNext(ifb);
        IIfBlock ifb1 = factory.createIfBlock("a > 1 ");
        ifb1.setYesBlock(factory.createActionBlock("a = a - 1 "));
        ifb1.setNoBlock(factory.createActionBlock("a = 1 - a "));
        ifb.setYesBlock(factory.createActionBlock("a = a - 1 "));
        ifb.setNoBlock(ifb1);
        b = factory.createActionBlock("a = a + 2 ");
        ifb.setNext(b);
        b.setNext(factory.createEndBlock("return"));
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                    testFolderPath + "TestSchemeWithNestedIf3.java",
                    "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            fail();
        }
    }
    
    public void testSchemeWithNestedIf4() {
        IBlock b = factory.createActionBlock("a = a + 1 ");
        scheme.getBeginBlock().setNext(b);
        IIfBlock ifb = factory.createIfBlock("a > 1 ");
        b.setNext(ifb);
        ifb.setYesBlock(factory.createActionBlock("a = a - 1 "));
        b = factory.createActionBlock("a = 1 - a ");
        ifb.setNoBlock(b);
        IIfBlock ifb1 = factory.createIfBlock("a > 1 ");
        ifb1.setYesBlock(factory.createActionBlock("a = a - 1 "));
        ifb1.setNoBlock(factory.createActionBlock("a = 1 - a "));
        b.setNext(ifb1);
        b = factory.createActionBlock("a = a + 2 ");
        ifb.setNext(b);
        b.setNext(factory.createEndBlock("return"));
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                    testFolderPath + "TestSchemeWithNestedIf4.java",
                    "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            fail();
        }
    }
    
    public void testSchemeWithTwoNestedIf() {
        IBlock b = factory.createActionBlock("a = a + 1 ");
        scheme.getBeginBlock().setNext(b);
        IIfBlock ifb = factory.createIfBlock("a > 1 ");
        b.setNext(ifb);
        IIfBlock ifb1 = factory.createIfBlock("a > 2 ");
        ifb.setYesBlock(ifb1);
        ifb1.setYesBlock(factory.createActionBlock("a = a + 2 "));
        ifb1.setNoBlock(factory.createActionBlock("a = a + 3 "));
        IIfBlock ifb2 = factory.createIfBlock("a > 3 ");
        ifb1.setNext(ifb2);
        ifb2.setYesBlock(factory.createActionBlock("a = a + 2 "));
        ifb2.setNoBlock(factory.createActionBlock("a = a + 3 "));
        b = factory.createActionBlock("a = a + 2 ");
        ifb.setNext(b);
        b.setNext(factory.createEndBlock("return"));
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                    testFolderPath + "TestSchemeWithTwoNestedIf.java",
                    "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            fail();
        }
        
    }
    
    public void testSchemeWithDoubleNestedIf() {
        IBlock b = factory.createActionBlock("a = a + 1 ");
        scheme.getBeginBlock().setNext(b);
        IIfBlock ifb = factory.createIfBlock("a > 1 ");
        b.setNext(ifb);
        b = factory.createActionBlock("a = a + 2 ");
        ifb.setNext(b);
        b.setNext(factory.createEndBlock("return"));
        IIfBlock ifb1 = factory.createIfBlock("a > 2 ");
        ifb.setYesBlock(ifb1);
        ifb = factory.createIfBlock("a > 3 ");
        ifb1.setYesBlock(ifb);
        try {
            assertEquals(scheme.toString(), new SchemeBuilder(new Method(
                    testFolderPath + "TestSchemeWithDoubleNestedIf.java",
                    "test ( ) ")).build().toString());
        } catch (SchemeBuilderException e) {
            fail();
        }
    }


}
