package test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;

import junit.framework.TestCase;

import org.amse.ak.schemebuilder.model.*;
import org.amse.ak.schemebuilder.model.impl.BlockFactory;
import org.amse.ak.schemebuilder.model.impl.Method;
import org.amse.ak.schemebuilder.model.impl.Scheme;

public class ModelTests extends TestCase {

    IScheme scheme;

    IBlockFactory factory;

    public void setUp() {
        scheme = new Scheme(new Method(null, "int test(int a, int b)"));
        factory = new BlockFactory();
    }

    private String readStringFromFile(String testName) {
        Reader reader = null;
        String fileName = "";
        try {
            fileName = "test" + File.separator + "test" + testName + ".txt";
            reader = new BufferedReader(new FileReader(fileName));
        } catch (FileNotFoundException e) {
            fail(fileName + " not found");
        }

        StringBuffer sb = new StringBuffer();
        try {
            try {
                char[] buffer = new char[1024];
                int counter;
                while ((counter = reader.read(buffer)) != -1) {
                    sb.append(new String(buffer, 0, counter));
                }
            } finally {
                reader.close();
            }
        } catch (IOException e) {

        }
        return sb.toString();
    }

    public void testBlockFactoryAction() {
        assertEquals(readStringFromFile("BlockFactory"),
                factory.createActionBlock("test").toString());
    }

    public void testBlockFactoryBegin() {
        assertEquals(readStringFromFile("BlockFactory"),
                factory.createBeginBlock("test").toString());
    }

    public void testBlockFactoryEnd() {
        assertEquals(readStringFromFile("BlockFactory"),
                factory.createEndBlock("test").toString());
    }

    public void testBlockFactoryFor() {
        assertEquals(readStringFromFile("BlockFactory"),
                factory.createForBlock("test").toString());
    }

    public void testBlockFactoryIf() {
        assertEquals(readStringFromFile("BlockFactory"),
                factory.createIfBlock("test").toString());
    }

    public void testBlockFactorySwitch() {
        assertEquals(readStringFromFile("BlockFactory"), factory
                .createSwitchBlock("test").toString());
    }

    public void testEmptyScheme() {
        String sample = readStringFromFile("EmptyScheme");
        assertEquals(sample, scheme.toString());
        assertEquals(sample, scheme.getBeginBlock().toString());
    }

    public void testGetBeginBlockOnNotEmptyScheme() {
        IBlock b = factory.createActionBlock("a = a + 1");
        scheme.getBeginBlock().setNext(b);
        b.setNext(factory.createEndBlock("return 0"));
        assertEquals(readStringFromFile("GetBeginBlockOnNotEmptyScheme"),
                scheme.getBeginBlock().toString());
    }

    public void testSchemeWithEndBlock() {
        scheme.getBeginBlock().setNext(factory.createEndBlock("return 0"));
        assertEquals(readStringFromFile("SchemeWithEndBlock"),
                scheme.toString());
    }

    public void testSchemeWithActionBlock() {
        IBlock b = factory.createActionBlock("a = a + 1");
        scheme.getBeginBlock().setNext(b);
        b.setNext(factory.createEndBlock("return 0"));
        assertEquals(readStringFromFile("SchemeWithActionBlock"),
                scheme.toString());
    }

    public void testSchemeWithFor1() {
        IBlock b = factory.createActionBlock("a = a + 1");
        scheme.getBeginBlock().setNext(b);
        IForBlock forb = factory.createForBlock("int i = 0; i < a; ++i");
        b.setNext(forb);
        forb.setNext(factory.createEndBlock("return 0"));
        assertEquals(readStringFromFile("SchemeWithFor1"), scheme.toString());
    }

    public void testSchemeWithFor2() {
        IForBlock forb = factory.createForBlock("int i = 0; i < a; ++i");
        scheme.getBeginBlock().setNext(forb);
        IBlock b = factory.createActionBlock("b = b + 1");
        forb.setNext(b);
        b.setNext(factory.createEndBlock("return 0"));
        assertEquals(readStringFromFile("SchemeWithFor2"), scheme.toString());
    }

    public void testSchemeWithFor3() {
        IBlock b = factory.createActionBlock("a = a + 1");
        scheme.getBeginBlock().setNext(b);
        IForBlock forb = factory.createForBlock("int i = 0; i < a; ++i");
        b.setNext(forb);
        forb.setFirstBlock(factory.createActionBlock("a = a + 1"));
        b = factory.createActionBlock("b = b + 1");
        forb.setNext(b);
        b.setNext(factory.createEndBlock("return 0"));
        assertEquals(readStringFromFile("SchemeWithFor3"), scheme.toString());
    }

    public void testSchemeWithFor4() {
        IBlock b = factory.createActionBlock("a = a + 1");
        scheme.getBeginBlock().setNext(b);
        IForBlock forb = factory.createForBlock("int i = 0; i < a; ++i");
        b.setNext(forb);
        IForBlock forb2 = factory.createForBlock("int j = 0; j < b; ++j");
        forb.setFirstBlock(forb2);
        forb2.setFirstBlock(factory.createActionBlock("b = b + 1"));
        forb2.setNext(factory.createActionBlock("b = b / 2"));
        b = factory.createActionBlock("b = b + 1");
        forb.setNext(b);
        b.setNext(factory.createEndBlock("return 0"));
        assertEquals(readStringFromFile("SchemeWithFor4"), scheme.toString());
    }

    public void testSchemeWithIf1() {
        IBlock b = factory.createActionBlock("a = a + 1");
        scheme.getBeginBlock().setNext(b);
        IIfBlock ifb = factory.createIfBlock("a > b");
        b.setNext(ifb);
        ifb.setNext(factory.createEndBlock("return 0"));
        assertEquals(readStringFromFile("SchemeWithIf1"), scheme.toString());
    }

    public void testSchemeWithIf2() {
        IIfBlock ifb = factory.createIfBlock("a > b");
        scheme.getBeginBlock().setNext(ifb);
        IBlock b = factory.createActionBlock("b = b + 1");
        ifb.setNext(b);
        b.setNext(factory.createEndBlock("return 0"));
        assertEquals(readStringFromFile("SchemeWithIf2"), scheme.toString());
    }

    public void testSchemeWithIf3() {
        IBlock b = factory.createActionBlock("a = a + 1");
        scheme.getBeginBlock().setNext(b);
        IIfBlock ifb = factory.createIfBlock("a > b");
        b.setNext(ifb);
        ifb.setYesBlock(factory.createActionBlock("a = a - b"));
        ifb.setNoBlock(factory.createActionBlock("a = b - a"));
        b = factory.createActionBlock("b = b + 1");
        ifb.setNext(b);
        b.setNext(factory.createEndBlock("return 0"));
        assertEquals(readStringFromFile("SchemeWithIf3"), scheme.toString());
    }

    public void testSchemeWithIf4() {
        IBlock b = factory.createActionBlock("a = a + 1");
        scheme.getBeginBlock().setNext(b);
        IIfBlock ifb = factory.createIfBlock("a > b");
        b.setNext(ifb);
        IIfBlock ifb2 = factory.createIfBlock("a > 2 * b");
        ifb.setYesBlock(ifb2);
        ifb2.setYesBlock(factory.createActionBlock("a = a - 2 * b"));
        ifb2.setNoBlock(factory.createActionBlock("a = 2 * b - a"));
        ifb2.setNext(factory.createActionBlock("a = a * a"));
        ifb.setNoBlock(factory.createActionBlock("a = b - a"));
        b = factory.createActionBlock("b = b + 1");
        ifb.setNext(b);
        b.setNext(factory.createEndBlock("return 0"));
        assertEquals(readStringFromFile("SchemeWithIf4"), scheme.toString());
    }

    public void testMethodWithoutParameters() {
        IMethod method = new Method("test.java", "testMethod ( ) ");
        assertEquals(readStringFromFile("MethodWithoutParameters"), method
                .toString());
        assertEquals("test.java", method.getClassPath());
        assertEquals("testMethod ( ) ", method.getName());
    }

    public void testMethodWithParameters() {
        IMethod method = new Method("test.java", "testMethod (int n , double d ) ");
        assertEquals(readStringFromFile("MethodWithParameters"), method
                .toString());
        assertEquals("test.java", method.getClassPath());
        assertEquals("testMethod (int n , double d ) ", method.getName());
    }

}
