package test;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Random;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

import org.amse.ak.schemebuilder.io.writer.SchemeWriter;
import org.amse.ak.schemebuilder.model.IBlock;
import org.amse.ak.schemebuilder.model.IBlockFactory;
import org.amse.ak.schemebuilder.model.IDoBlock;
import org.amse.ak.schemebuilder.model.IForBlock;
import org.amse.ak.schemebuilder.model.IIfBlock;
import org.amse.ak.schemebuilder.model.IScheme;
import org.amse.ak.schemebuilder.model.ISwitchBlock;
import org.amse.ak.schemebuilder.model.ITryBlock;
import org.amse.ak.schemebuilder.model.IWhileBlock;
import org.amse.ak.schemebuilder.model.impl.BlockFactory;
import org.amse.ak.schemebuilder.model.impl.Method;
import org.amse.ak.schemebuilder.model.impl.Scheme;
import org.amse.ak.schemebuilder.view.MyFrame;

public class ViewTests {

    public static final int MAX_NESTING = 4;

    public static final int MAX_DEPTH = 2;
    
    public static final int TYPE = 6;
    
    public static final int TEST_NUMBER = 100;
    
    static private final IBlockFactory myFactory = new BlockFactory();
    
    static private final Random myRandom = new Random(); 
    
    static public IBlock createActionBlocks(int number, String text) {
        IBlock b = myFactory.createActionBlock(text);
        IBlock res = b;
        for (int i = 1; i < number; i++) {
            b.setNext(myFactory.createActionBlock(text));
            b = b.getNext();
        }
        return res;
    }
    
    
    static public void createCompoundBlock(int nesting, boolean isFirstCall, IBlock current) {
    	System.out.println("Compound nesting="+nesting);
        //    .
        int nextNesting = nesting - 1;
        //   .
        if (isFirstCall) {
            nextNesting = MAX_NESTING;//myRandom.nextInt(nesting + 1);
        }
        //   
        int type = myRandom.nextInt(TYPE);
        switch (type) {
        case 0:
            IIfBlock ib = myFactory.createIfBlock("If Block");
            ib.setYesBlock(createRandomScheme(nextNesting, false));
            ib.setNoBlock(createRandomScheme(nextNesting, false));
            current.setNext(ib);
            break;
        case 1:
            IForBlock fb = myFactory.createForBlock("For Block");
            fb.setFirstBlock(createRandomScheme(nextNesting, false));
            current.setNext(fb);
            break;
        case 2:
            IWhileBlock wb = myFactory.createWhileBlock("While Block");
            wb.setFirstBlock(createRandomScheme(nextNesting, false));
            current.setNext(wb);
            break;
        case 3:
            IDoBlock dwb = myFactory.createDoBlock("Do Block");
            dwb.setFirstBlock(createRandomScheme(nextNesting, false));
            current.setNext(dwb);
            break;
        case 4:
            ISwitchBlock sb = myFactory.createSwitchBlock("Switch Block");
            sb.addCase("1", createRandomScheme(nextNesting, false));
            sb.addCase("2", createRandomScheme(nextNesting, false));
            current.setNext(sb);
            break;
        case 5:
            ITryBlock tb = myFactory.createTryBlock();
            IBlock cb = myFactory.createActionBlock("catch 1"); 
            tb.addCatch(cb);
            cb.setNext(createRandomScheme(nextNesting, false));
            cb = myFactory.createActionBlock("catch 2"); 
            tb.addCatch(cb);
            cb.setNext(createRandomScheme(nextNesting, false));
            current.setNext(tb);
            break;
        }
    }
    
    static public IBlock createRandomScheme(int nesting, boolean isFirstCall) {
        //     .    		
    	
        if (nesting == 0) {
            return createActionBlocks(myRandom.nextInt(MAX_DEPTH), "Action Block");
        //   ,      .
        } else if (myRandom.nextBoolean()) {
            IBlock root = createActionBlocks(myRandom.nextInt(MAX_DEPTH), "Action Block");
            IBlock current = root;
            while (current.getNext() != null) {
                current = current.getNext();
            }
            //   .
            //int n = myRandom.nextInt(MAX_DEPTH);
            int n = 1;
            for (int i = 0; i < n; i++) {
                createCompoundBlock(nesting, isFirstCall, current);
                current = current.getNext();
            }
            return root;
        //     .
        } else {
            IBlock root = myFactory.createActionBlock("Root");
            IBlock current = root;
            //   .
            int n = myRandom.nextInt(MAX_DEPTH);
            for (int i = 0; i < n; i++) {
                createCompoundBlock(nesting, isFirstCall, current);
                current = current.getNext();
            }
            current.setNext(createActionBlocks(myRandom.nextInt(MAX_DEPTH), "Action Block"));
            return root.getNext();
        }
    }
    
    public static void main (String[] args) {
        for (int i = 0; i < TEST_NUMBER; i++) {
            System.out.println("" + i);
            IScheme scheme = new Scheme(new Method("", "Random Scheme " + i));
            scheme.getBeginBlock().setNext(createRandomScheme(myRandom.nextInt(MAX_NESTING), true));
            MyFrame frame = new MyFrame(null, 0);
            frame.setScheme(scheme);
            JPanel schemePanel = frame.getSchemePanel(); 
            Dimension d = schemePanel.getPreferredSize();            
            System.out.println(d.getWidth() + " " + d.getHeight());
            BufferedImage image = new BufferedImage((int) Math.round(d.getWidth()),
                    (int) Math.round(d.getHeight()), BufferedImage.TYPE_INT_RGB);
            frame.setSize(640, 480);
            frame.pack();            
            
            Graphics g = image.createGraphics();
            schemePanel.paint(g);
            
            String si = "" + i;
            if (i < 10) {
                si = "000" + i;
            } else if (i < 100) {
                si = "00" + i;
            } else if (i < 1000) {
                si = "0" + i;
            }
            try {
                File f = new File("test" + File.separator + "Random Scheme " + si + ".jpg");
                ImageIO.write(image, "jpg", f);
            } catch (IOException exception) {
                System.out.println("Couldn't export scheme to jpg.");
            }
            scheme.setFile(new File("test" + File.separator + "Random Scheme " + si + ".sch"));
            try {
                SchemeWriter fw = new SchemeWriter(scheme);
                fw.write();
              } catch (Exception e) {
                  e.printStackTrace();
                  System.out.println("Couldn't write scheme.");
              }
            System.out.println("End.");
            frame.dispose();
        }
        

    }
    
}